@@ -25,6 +25,11 @@ static const gchar developers[] =
     "Luke Somers\n"
     "Vojtěch Salajka\n"
     "Anna Campbellova\n"
+    "Lennart Fricke\n"
+    "Jozef Veselý\n"
+    "Gianfranco Gallizia\n"
+    "Sameer Grover\n"
+    "Niv Levy\n"
 ;
 static const gchar translators[] =
     "David Nečas (Yeti)\n"
@@ -36,3 +41,6 @@ static const gchar translators[] =
     "Даниил Браташов\n"
     "Andrés Muñiz Piniella\n"
 ;
+static const gchar releasedate[] =
+    "2013-09-08"
+;
