@@ -1,6 +1,6 @@
 /*
- *  @(#) $Id: gwyappinternal.h 11810 2011-01-21 05:48:12Z klapetek $
- *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
+ *  @(#) $Id: gwyappinternal.h 15022 2013-05-12 20:12:49Z yeti-dn $
+ *  Copyright (C) 2004,2013 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 /*< private_header >*/
@@ -25,6 +25,7 @@
 
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libprocess/spectra.h>
+#include <libprocess/brick.h>
 #include <libgwydgets/gwydatawindow.h>
 #include <libgwydgets/gwy3dwindow.h>
 #include <libgwydgets/gwygraphwindow.h>
@@ -45,7 +46,7 @@ typedef enum {
     KEY_IS_GRAPH_VISIBLE,
     KEY_IS_SPECTRA,
     KEY_IS_SPECTRA_VISIBLE,
-    KEY_IS_META,
+    KEY_IS_CHANNEL_META,
     KEY_IS_TITLE,
     KEY_IS_SELECT,
     KEY_IS_RANGE_TYPE,
@@ -60,7 +61,13 @@ typedef enum {
     KEY_IS_SPS_REF,
     KEY_IS_FILENAME,
     KEY_IS_CALDATA,
-    KEY_IS_GRAPH_LASTID
+    KEY_IS_GRAPH_LASTID,
+    KEY_IS_BRICK,
+    KEY_IS_BRICK_VISIBLE,
+    KEY_IS_BRICK_TITLE,
+    KEY_IS_BRICK_PREVIEW,
+    KEY_IS_BRICK_PREVIEW_PALETTE,
+    KEY_IS_BRICK_META,
 } GwyAppKeyType;
 
 G_GNUC_INTERNAL
@@ -75,12 +82,12 @@ gboolean _gwy_app_3d_view_init_setup          (GwyContainer *container,
                                                const gchar *setup_prefix);
 G_GNUC_INTERNAL
 void     _gwy_app_graph_window_setup          (GwyGraphWindow *graph_window);
+G_GNUC_INTERNAL
+void     _gwy_app_brick_window_setup          (GwyDataWindow *data_window);
 
 G_GNUC_INTERNAL
 void     _gwy_app_data_view_set_current       (GwyDataView *data_view);
 G_GNUC_INTERNAL
-void     _gwy_app_graph_set_current           (GwyGraph *graph);
-G_GNUC_INTERNAL
 void     _gwy_app_spectra_set_current         (GwySpectra *spectra);
 
 G_GNUC_INTERNAL
