@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyddion.c 11803 2011-01-19 22:33:27Z yeti-dn $
+ *  @(#) $Id: gwyddion.c 15053 2013-05-23 14:27:06Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include "config.h"
@@ -42,6 +42,7 @@
 #define LOG_TO_FILE_DEFAULT TRUE
 #include <windows.h>
 #include <winreg.h>
+#define gwyddion_key "Software\\Gwyddion\\2.0"
 #else
 #define LOG_TO_FILE_DEFAULT FALSE
 #endif
@@ -91,12 +92,15 @@ main(int argc, char *argv[])
 {
     GtkWidget *toolbox;
     gchar **module_dirs;
-    gchar *settings_file, *recent_file_file;
+    gchar *settings_file, *recent_file_file, *accel_file;
     gboolean has_settings, settings_ok = FALSE;
     GError *settings_err = NULL;
     GTimer *timer;
 
     sneaking_thread_init();
+    // If I print a debugging messages that's because some wants to see it
+    // you idiots.
+    g_setenv("G_MESSAGES_DEBUG", "all", TRUE);
     timer = g_timer_new();
     gwy_app_check_version();
 
@@ -125,6 +129,10 @@ main(int argc, char *argv[])
     gwy_app_splash_start(!app_options.no_splash && !app_options.check);
     debug_time(timer, "create splash");
 
+    accel_file = g_build_filename(gwy_get_user_dir(), "ui", "accel_map", NULL);
+    gtk_accel_map_load(accel_file);
+    debug_time(timer, "load accel map");
+
     gwy_app_splash_set_message(_("Loading document history"));
     recent_file_file = gwy_app_settings_get_recent_file_list_filename();
     gwy_app_recent_file_list_load(recent_file_file);
@@ -159,6 +167,8 @@ main(int argc, char *argv[])
     /* The Python initialisation somehow overrides SIGINT and Gwyddion can no
      * longer be terminated with Ctrl-C.  Fix it. */
     signal(SIGINT, SIG_DFL);
+    /* TODO: The Python initialisation also overrides where the warnings go.
+     * Restore the handlers. */
     debug_time(timer, "register modules");
 
     if (app_options.check) {
@@ -208,6 +218,7 @@ main(int argc, char *argv[])
     /* TODO: handle failure */
     if (settings_ok || !has_settings)
         gwy_app_settings_save(settings_file, NULL);
+    gtk_accel_map_save(accel_file);
     debug_time(timer, "save settings");
     gwy_app_recent_file_list_save(recent_file_file);
     debug_time(timer, "save document history");
@@ -224,6 +235,7 @@ main(int argc, char *argv[])
      * Remove in production version. */
     g_free(recent_file_file);
     g_free(settings_file);
+    g_free(accel_file);
     g_strfreev(module_dirs);
     debug_time(timer, "destroy resources");
     g_timer_destroy(timer);
@@ -404,7 +416,7 @@ setup_logging(void)
         "GLib", "GLib-GObject", "GLib-GIO", "GModule", "GThread",
         "GdkPixbuf", "Gdk", "Gtk",
         "GdkGLExt", "GtkGLExt",
-        "Pango",
+        "Pango", "Unique",
         "Gwyddion", "GwyProcess", "GwyDraw", "Gwydgets", "GwyModule", "GwyApp",
         "Module", NULL
     };
@@ -458,47 +470,91 @@ static void
 setup_locale_from_win32_registry(void)
 {
 #ifdef G_OS_WIN32
-    gchar locale[64];
-    DWORD size = sizeof(locale);
+    gchar locale[65];
+    DWORD size = sizeof(locale)-1;
     HKEY reg_key;
 
-    if (RegOpenKeyEx(HKEY_CURRENT_USER, TEXT("Software\\Gwyddion\\2.0"),
+    gwy_clear(locale, sizeof(locale));
+    if (RegOpenKeyEx(HKEY_CURRENT_USER, TEXT(gwyddion_key),
                      0, KEY_READ, &reg_key) == ERROR_SUCCESS) {
-        if (RegQueryValueEx(reg_key, TEXT("Locale"), NULL, NULL, locale, &size) == ERROR_SUCCESS){
+        if (RegQueryValueEx(reg_key, TEXT("Locale"), NULL, NULL,
+                            locale, &size) == ERROR_SUCCESS) {
             g_setenv("LANG", locale, TRUE);
             RegCloseKey(reg_key);
             return;
         }
         RegCloseKey(reg_key);
     }
-    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, TEXT("Software\\Gwyddion\\2.0"),
+    if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, TEXT(gwyddion_key),
                      0, KEY_READ, &reg_key) == ERROR_SUCCESS) {
-        if (RegQueryValueEx(reg_key, TEXT("Locale"), NULL, NULL, locale, &size) == ERROR_SUCCESS)
+        if (RegQueryValueEx(reg_key, TEXT("Locale"), NULL, NULL,
+                            locale, &size) == ERROR_SUCCESS)
             g_setenv("LANG", locale, TRUE);
         RegCloseKey(reg_key);
     }
 #endif
 }
 
+static gchar*
+fix_win32_commandline_arg(gchar *p)
+{
+#ifdef G_OS_WIN32
+    int buflen, n = strlen(p);
+    gchar *q;
+    wchar_t *w;
+
+    if (!(buflen = MultiByteToWideChar(CP_ACP, 0, p, n, NULL, 0)))
+        return g_strdup(p);
+    w = g_new(wchar_t, buflen+1);
+    if (!MultiByteToWideChar(CP_ACP, 0, p, n, w, buflen+1)) {
+        /* XXX: This should not really happen. */
+        g_free(w);
+        return g_strdup(p);
+    }
+    w[buflen] = 0;
+    if (!(q = g_utf16_to_utf8((const gunichar2*)w, buflen, NULL, NULL, NULL))) {
+        g_free(w);
+        return g_strdup(p);
+    }
+    g_free(w);
+    return q;
+#else
+    return g_strdup(p);
+#endif
+}
+
 static void
 open_command_line_files(gint n, gchar **args)
 {
     gchar **p;
-    gchar *cwd, *filename;
+    gchar *cwd, *filename, *q;
 
-    /* FIXME: cwd is in GLib encoding. And args? */
     cwd = g_get_current_dir();
+#ifdef DEBUG
+    gwy_debug("current dir: <%s>", g_strescape(cwd, ""));
+#endif
     for (p = args; n; p++, n--) {
-        if (g_path_is_absolute(*p))
-            filename = g_strdup(*p);
+        q = fix_win32_commandline_arg(*p);
+#ifdef DEBUG
+        gwy_debug("argv: <%s>", g_strescape(*p, ""));
+        gwy_debug("converted: <%s>", g_strescape(q, ""));
+#endif
+        if (g_path_is_absolute(q))
+            filename = g_strdup(q);
         else
-            filename = g_build_filename(cwd, *p, NULL);
+            filename = g_build_filename(cwd, q, NULL);
+
         if (g_file_test(filename, G_FILE_TEST_IS_DIR)) {
             gwy_app_set_current_directory(filename);
             gwy_app_file_open();
         }
-        else
+        else {
+#ifdef DEBUG
+            gwy_debug("calling gwy_app_file_load() with <%s>", g_strescape(filename, ""));
+#endif
             gwy_app_file_load(NULL, filename, NULL);
+        }
+        g_free(q);
         g_free(filename);
     }
     g_free(cwd);
@@ -628,6 +684,10 @@ sneaking_thread_init(void)
         gwy_debug("GLib is not >= 2.24.0; says %s", version_mismatch);
         return;
     }
+    if (glib_check_version(2, 32, 0)) {
+        gwy_debug("GLib is >= 2.32.0");
+        return;
+    }
 
     main_module = g_module_open(NULL, G_MODULE_BIND_LAZY | G_MODULE_BIND_LOCAL);
     if (!main_module) {
