@@ -1,6 +1,6 @@
 
 /*
- *  @(#) $Id: gwyfdcurvepreset.c 9045 2009-03-10 12:00:57Z yeti-dn $
+ *  @(#) $Id: gwyfdcurvepreset.c 15049 2013-05-16 21:01:44Z klapetek $
  *  Copyright (C) 2000-2003 Martin Siler.
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
@@ -17,7 +17,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include <libgwyddion/gwymath.h>
@@ -331,6 +331,39 @@ parcapella_guess(gint n_dat,
     *fres = TRUE;
 }
 
+/******************* hertz paraboloid ********************************/
+static gdouble
+hertzpar_func(gdouble x,
+                G_GNUC_UNUSED gint n_param,
+                const gdouble *b,
+                G_GNUC_UNUSED gpointer user_data,
+                gboolean *fres)
+{
+    /*xc, R, E */
+    *fres = TRUE;
+    if ((b[0]-x)>0) return 1.3333333*b[2]*sqrt(b[1]*(b[0]-x)*(b[0]-x)*(b[0]-x));
+    else return 0;
+}
+
+static void
+hertzpar_guess(gint n_dat,
+                 const gdouble *x,
+                 const gdouble *y,
+                 gdouble *param,
+                 gboolean *fres)
+{
+    gint i;
+    gdouble xmax = x[n_dat - 1];
+
+    for (i = 1; i < n_dat; i++) {
+        if (x[i] > xmax) xmax = x[i];
+    }
+    param[0] = xmax;
+    param[1] = 10e-9;
+    param[2] = 10e6;
+
+    *fres = TRUE;
+}
 
 
 /******************* sphtiptap ********************************/
@@ -738,6 +771,12 @@ static const GwyNLFitParam hsphhertz_params[] = {
     { "nu", 1, 0, },
 };
 
+static const GwyNLFitParam hertzpar_params[] = {
+    { "xc", 1, 0, },
+    { "R", -1, 0, },
+    { "Er", -1, 1, },
+};
+
 
 
 static const GwyNLFitPresetBuiltin fitting_presets[] = {
@@ -859,6 +898,19 @@ static const GwyNLFitPresetBuiltin fitting_presets[] = {
         NULL,
         G_N_ELEMENTS(parcapella_params),
         parcapella_params,
+    },
+     {
+        N_("contact: Hertz (paraboloid)"),
+        "<i>f</i>(<i>x</i>) "
+        "= 4/3 E sqrt(R(<i>x</i>-<i>x<sub>c</sub></i>)<sup>3</sup>) ",
+        &hertzpar_func,
+        NULL,
+        &hertzpar_guess,
+        NULL,
+        NULL,
+        NULL,
+        G_N_ELEMENTS(hertzpar_params),
+        hertzpar_params,
     },
      /*    {
         "vdW: sphere3",
