@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwynlfit.c 8874 2008-12-28 12:16:22Z yeti-dn $
+ *  @(#) $Id: gwynlfit.c 15041 2013-05-14 20:05:09Z klapetek $
  *  Copyright (C) 2000-2003 Martin Siler.
  *  Copyright (C) 2004,2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include "config.h"
@@ -422,7 +422,7 @@ gwy_math_nlfit_fit_full(GwyNLFitter *nlfit,
         }
         else {
             /* XXX: else what? */
-            g_warning("Cannot invert covariance matrix");
+            //g_warning("Cannot invert covariance matrix");
             sumr = -1.0;
         }
     }
