@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyserializable.c 10097 2009-09-03 08:47:14Z yeti-dn $
+ *  @(#) $Id: gwyserializable.c 13351 2012-03-29 09:23:12Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -15,13 +15,11 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include "config.h"
 #include <string.h>
-#include <glib-object.h>
-#include <glib/gutils.h>
 
 #include <libgwyddion/gwymacros.h>
 #include <libgwyddion/gwyutils.h>
