@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyversion.h.in 7355 2007-01-13 23:36:11Z yeti-dn $
+ *  @(#) $Id: gwyversion.h.in 13351 2012-03-29 09:23:12Z yeti-dn $
  *  Copyright (C) 2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -15,19 +15,19 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #ifndef __GWY_VERSION_H__
 #define __GWY_VERSION_H__
 
-#include <glib/gtypes.h>
+#include <glib.h>
 
 G_BEGIN_DECLS
 
 #define GWY_VERSION_MAJOR 2
-#define GWY_VERSION_MINOR 25
-#define GWY_VERSION_STRING "2.25"
+#define GWY_VERSION_MINOR 32
+#define GWY_VERSION_STRING "2.32"
 
 gint gwy_version_major(void);
 gint gwy_version_minor(void);
