@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwy3dlabel.c 8884 2008-12-28 12:17:44Z yeti-dn $
+ *  @(#) $Id: gwy3dlabel.c 15299 2013-09-08 06:36:08Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include "config.h"
@@ -573,7 +573,8 @@ gwy_3d_label_reset(Gwy3DLabel *label)
         label->delta_y = defval;
         g_object_notify(object, "delta-y");
     }
-    if (label->rotation != (defval = PSPEC("rotation", DOUBLE)->default_value)) {
+    if (label->rotation
+        != (defval = PSPEC("rotation", DOUBLE)->default_value)) {
         label->rotation = defval;
         g_object_notify(object, "rotation");
     }
@@ -630,7 +631,7 @@ gwy_3d_label_user_size(Gwy3DLabel *label,
 /**
  * SECTION:gwy3dlabel
  * @title: Gwy3DLabel
- * @short_description: Label on #Gwy3DView
+ * @short_description: Label on Gwy3DView
  **/
 
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
