@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwydgetutils.c 9629 2009-08-20 12:41:20Z yeti-dn $
+ *  @(#) $Id: gwydgetutils.c 13903 2012-10-08 06:51:16Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include "config.h"
@@ -561,8 +561,6 @@ gwy_mask_color_selector_run(const gchar *dialog_title,
     dialog = gtk_color_selection_dialog_new(dialog_title
                                             ? dialog_title
                                             : _("Change Mask Color"));
-    if (gtk_major_version == 2 && gtk_minor_version < 10)
-        gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
     selector = GTK_COLOR_SELECTION_DIALOG(dialog)->colorsel;
     gtk_color_selection_set_current_color(GTK_COLOR_SELECTION(selector),
                                           &gdkcolor);
