@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyshader.c 6343 2006-07-15 21:35:45Z yeti-dn $
+ *  @(#) $Id: gwyshader.c 14843 2013-04-10 10:26:27Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include "config.h"
@@ -537,6 +537,10 @@ gwy_shader_realize(GtkWidget *widget)
                                           &attributes, attributes_mask);
     gdk_window_set_user_data(shader->event_window, shader);
 
+    /* Force widget to listen to relevant events.  Works around oxygen
+     * misdetection of ‘empty’ areas.  See KDE bug #317292. */
+    gtk_widget_add_events(widget, attributes.event_mask);
+
     widget->style = gtk_style_attach(widget->style, widget->window);
 
     gwy_shader_make_pixmap(shader);
