@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwymodule.h 8259 2007-06-25 16:55:54Z yeti-dn $
+ *  @(#) $Id: gwymodule.h 14845 2013-04-10 18:37:04Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #ifndef __GWY_MODULE_H__
@@ -29,6 +29,7 @@
 #include <libgwymodule/gwymodule-layer.h>
 #include <libgwymodule/gwymodule-process.h>
 #include <libgwymodule/gwymodule-tool.h>
+#include <libgwymodule/gwymodule-volume.h>
 #include <libgwymodule/gwymodulebrowser.h>
 
 #endif /* __GWY_MODULE_H__ */
