@@ -1,5 +1,5 @@
 /*
- *  $Id: apefile.c 10195 2009-10-01 23:04:26Z yeti-dn $
+ *  $Id: apefile.c 14701 2013-02-24 17:21:50Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -17,13 +17,13 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 /**
  * [FILE-MAGIC-FREEDESKTOP]
  * <mime-type type="application/x-ape-spm">
- *   <comment>APE SPM data</comment>
+ *   <comment>A.P.E. Research SPM data</comment>
  *   <magic priority="80">
  *     <match type="string" offset="234" value="APERES"/>
  *   </magic>
@@ -41,9 +41,19 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-FILEMAGIC]
+ * # A.P.E. Research old SPM data (old DAT).
+ * # This detects version 2.x files since some x.  Not sure about which.  Older
+ * # versions lack the APERES and cannot be recognised reliably.
+ * 0 byte =2
+ * >3 leshort <5
+ * >>234 string APERES A.P.E. Research SPM data, DAT version 2
+ **/
+
 /**
  * [FILE-MAGIC-USERGUIDE]
- * APE Research DAT
+ * A.P.E. Research DAT
  * .dat
  * Read
  **/
@@ -379,20 +389,64 @@ apefile_load(const gchar *filename,
         g_object_unref(apefile.data[n]);
 
         g_snprintf(key, sizeof(key), "/%d/data/title", n);
-        title = gwy_enuml_to_string(i,
-                                    "Height",   APE_HEIGHT,
-                                    "Height-R", APE_HEIGHT_R,
-                                    "NSOM",     APE_NSOM,
-                                    "NSOM-R",   APE_NSOM_R,
-                                    "Error",    APE_ERROR,
-                                    "Error-R",  APE_ERROR_R,
-                                    "NSOM2",    APE_NSOM2,
-                                    "NSOM2-R",  APE_NSOM2_R,
-                                    "Aux1",     APE_AUX1,
-                                    "Aux2",     APE_AUX2,
-                                    "Aux1-R",   APE_AUX1_R,
-                                    "Aux2-R",   APE_AUX2_R,
-                                    NULL);
+
+        /*
+         * Channel labelling based on SPM Mode
+         */
+        switch(apefile.spm_mode) {
+            case SPM_MODE_SNOM:
+            title = gwy_enuml_to_string(i,
+                                        "Height",   APE_HEIGHT,
+                                        "Height-R", APE_HEIGHT_R,
+                                        "NSOM",     APE_NSOM,
+                                        "NSOM-R",   APE_NSOM_R,
+                                        "Error",    APE_ERROR,
+                                        "Error-R",  APE_ERROR_R,
+                                        "NSOM2",    APE_NSOM2,
+                                        "NSOM2-R",  APE_NSOM2_R,
+                                        "Lateral",     APE_AUX1,
+                                        "Z-Z0",     APE_AUX2,
+                                        "Lateral-R",   APE_AUX1_R,
+                                        "Z-Z0-R",   APE_AUX2_R,
+                                        NULL);
+            break;
+
+            case SPM_MODE_AFM_NONCONTACT:
+            case SPM_MODE_AFM_CONTACT:
+            case SPM_MODE_PHASE_DETECT_AFM:
+            title = gwy_enuml_to_string(i,
+                                        "Height",   APE_HEIGHT,
+                                        "Height-R", APE_HEIGHT_R,
+                                        "IN1",     APE_NSOM,
+                                        "IN1-R",   APE_NSOM_R,
+                                        "Error",    APE_ERROR,
+                                        "Error-R",  APE_ERROR_R,
+                                        "IN2",    APE_NSOM2,
+                                        "IN2-R",  APE_NSOM2_R,
+                                        "Lateral",     APE_AUX1,
+                                        "Z-Z0",     APE_AUX2,
+                                        "Lateral-R",   APE_AUX1_R,
+                                        "Z-Z0-R",   APE_AUX2_R,
+                                        NULL);
+            break;
+
+            default:
+            title = gwy_enuml_to_string(i,
+                                        "Height",   APE_HEIGHT,
+                                        "Height-R", APE_HEIGHT_R,
+                                        "IN1",     APE_NSOM,
+                                        "IN1-R",   APE_NSOM_R,
+                                        "Error",    APE_ERROR,
+                                        "Error-R",  APE_ERROR_R,
+                                        "IN2",    APE_NSOM2,
+                                        "IN2-R",  APE_NSOM2_R,
+                                        "Aux1",     APE_AUX1,
+                                        "Z-Z0",     APE_AUX2,
+                                        "Aux1-R",   APE_AUX1_R,
+                                        "Z-Z0-R",   APE_AUX2_R,
+                                        NULL);
+            break;
+        }
         if (title && *title)
             gwy_container_set_string_by_name(container, key, g_strdup(title));
 
@@ -519,7 +573,7 @@ apefile_get_metadata(APEFile *apefile)
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
  */
 
 typedef guint WORD;
