@@ -1,5 +1,5 @@
 /*
- *  $Id: gsffile.c 12307 2011-04-18 19:42:21Z yeti-dn $
+ *  $Id: gsffile.c 14705 2013-02-25 15:16:41Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 /**
@@ -29,6 +29,12 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-FILEMAGIC]
+ * # Gwyddion simple field files (GSF), see http://gwyddion.net/
+ * 0 string Gwyddion\ Simple\ Field\ 1.0\x0d\x0a Gwyddion simple field SPM data version 1.0
+ **/
+
 /**
  * [FILE-MAGIC-USERGUIDE]
  * Gwyddion Simple Field
@@ -296,7 +302,7 @@ gsf_export(GwyContainer *container,
            G_GNUC_UNUSED GwyRunType mode,
            GError **error)
 {
-    gchar zeroes[4] = { 0, 0, 0, 0 };
+    static const gchar zeroes[4] = { 0, 0, 0, 0 };
     GString *header = NULL;
     gfloat *dfl = NULL;
     guint i, xres, yres, padding;
