@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: miffile.c 11749 2011-01-12 07:43:49Z yeti-dn $
+ *  @(#) $Id: miffile.c 14711 2013-02-26 09:47:27Z yeti-dn $
  *  Copyright (C) 2010 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 /* TODO:
@@ -41,6 +41,12 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-FILEMAGIC]
+ * # Danish Micro Engineering MIF
+ * 0 string MIF\x01\x00 Danish Micro Engineering MIF SPM data
+ **/
+
 /**
  * [FILE-MAGIC-USERGUIDE]
  * DME MIF
@@ -596,7 +602,7 @@ mif_load(const gchar *filename,
         MIFInfoItem *item = mfile.images + i;
         MIFImageHeader image_header;
         MIFBlock raster, macro_geometry, preview, image, curve, calc;
-        guint ncalculations;
+        G_GNUC_UNUSED guint ncalculations;
         const guchar *p = buffer + item->image.offset;
         GQuark quark;
 
