@@ -1,5 +1,5 @@
 /*
- *  $Id: nrrdfile.c 12448 2011-05-22 21:07:33Z yeti-dn $
+ *  $Id: nrrdfile.c 14712 2013-02-26 14:04:32Z yeti-dn $
  *  Copyright (C) 2011 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 /**
@@ -30,6 +30,16 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-FILEMAGIC]
+ * # Nearly raw raster data
+ * # Metaformat, see http://teem.sourceforge.net/nrrd/format.html
+ * 0 string NRRD000 Nearly Raw Raster Data
+ * >&0 regex [0-9]+ version %s
+ * >&0 search/200 data\ file:\x20 text header for
+ * >>&0 regex [^\x0d\x0a]+ %s
+ **/
+
 /**
  * [FILE-MAGIC-USERGUIDE]
  * Nearly raw raster data (NRRD)
@@ -159,7 +169,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports and exports nearly raw raster data (NRRD) files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.1",
+    "0.2",
     "David Nečas (Yeti)",
     "2011",
 };
@@ -212,7 +222,8 @@ nrrdfile_load(const gchar *filename,
     GError *err = NULL;
     GwyDataField *dfield = NULL;
     guint sizes[3] = { 1, 1, 1 };
-    guint version, dimension, xres, yres, header_size, i,
+    G_GNUC_UNUSED guint version;
+    guint dimension, xres, yres, header_size, i,
           stride, rowstride, fieldstride, chanaxis, xaxis, yaxis, nchannels;
     gchar *value, *vkeyvalue, *vfield, *p, *line, *datafile = NULL;
     gchar **kinds = NULL;
@@ -675,7 +686,7 @@ load_detached_file(const gchar *datafile,
 
     if (gzcompressed) {
 #ifdef HAVE_ZLIB
-        gzFile *cfile = gzopen(datafile, "rb");
+        gzFile cfile = gzopen(datafile, "rb");
         gsize prevsize, toread;
         gssize readbytes;
 
@@ -1009,7 +1020,7 @@ fail:
                     (guint)(nitems-i), (guint)nitems);
     else
         g_set_error(error, GWY_MODULE_FILE_ERROR, GWY_MODULE_FILE_ERROR_DATA,
-                    _("Gargbage after data sample #%u."), (guint)(nitems-i));
+                    _("Garbage after data sample #%u."), (guint)(nitems-i));
 
     g_free(decoded);
     return NULL;
@@ -1322,7 +1333,7 @@ split_per_axis_field(const gchar *value,
                      GError **error)
 {
     gchar **raw_fields, **fields, *f;
-    guint n, i, len;
+    guint n = 0, i, len;
 
     raw_fields = g_strsplit_set(value, " \t\v\r\n", -1);
     fields = g_new0(gchar*, nitems+1);
@@ -1426,8 +1437,8 @@ nrrdfile_export(G_GNUC_UNUSED GwyContainer *data,
 
     g_ascii_formatd(buf[0], sizeof(buf[0]), "%.8g", xoff);
     g_ascii_formatd(buf[1], sizeof(buf[1]), "%.8g", yoff);
-    g_ascii_formatd(buf[2], sizeof(buf[2]), "%.8g", xreal - xoff);
-    g_ascii_formatd(buf[3], sizeof(buf[3]), "%.8g", yreal - yoff);
+    g_ascii_formatd(buf[2], sizeof(buf[2]), "%.8g", xreal + xoff);
+    g_ascii_formatd(buf[3], sizeof(buf[3]), "%.8g", yreal + yoff);
 
     /* Write in native endian. */
     fprintf(fh, header_format,
