@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: nt-mdt.c 12503 2011-06-04 07:53:33Z dn2010 $
+ *  @(#) $Id: nt-mdt.c 15294 2013-09-06 19:03:07Z dn2010 $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -15,9 +15,10 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
-/* TODO: some metadata, ... */
+
+/* TODO: some metadata ... */
 
 /**
  * [FILE-MAGIC-FREEDESKTOP]
@@ -31,6 +32,12 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-FILEMAGIC]
+ * # NT-MDT
+ * 0 belong 0x01b093ff NT-MDT SPM data
+ **/
+
 /**
  * [FILE-MAGIC-USERGUIDE]
  * NT-MDT
@@ -46,17 +53,14 @@
 #include <libgwyddion/gwymath.h>
 #include <libprocess/datafield.h>
 #include <libprocess/spectra.h>
-#include <libgwydgets/gwygraphmodel.h>
-#include <libgwydgets/gwygraphbasics.h>
+#include <libprocess/brick.h>
+#include <libgwydgets/gwydgets.h>
 #include <libgwymodule/gwymodule-file.h>
 #include <app/gwymoduleutils-file.h>
+#include <app/gwymoduleutils.h>
 
-#include <glib.h>
 #include <glib/gprintf.h>
 
-#include <gtk/gtk.h>
-#include <gdk-pixbuf/gdk-pixbuf.h>
-
 #include "err.h"
 #include "get.h"
 
@@ -74,6 +78,10 @@
 #define TREAT_CDATA_AS_TEXT 0
 #endif
 
+enum {
+    PREVIEW_SIZE = 200
+};
+
 typedef enum {
     MDT_FRAME_SCANNED      = 0,
     MDT_FRAME_SPECTROSCOPY = 1,
@@ -233,6 +241,12 @@ typedef enum {
     MDT_XML_DATAARRAY        = -1
 } MDTXMLParamType;
 
+typedef enum {
+    MDT_IMAGE_MAX,
+    MDT_IMAGE_MAXPOS,
+    MDT_IMAGE_AVG
+} MDTImageType;
+
 enum {
     FILE_HEADER_SIZE      = 32,
     FRAME_HEADER_SIZE     = 22,
@@ -374,19 +388,34 @@ typedef struct {
 } MDTXMLParams;
 
 typedef struct {
-    gint headersize;
-    gint coordsize;
-    gint version;
+    gint    headersize;
+    gint    coordsize;
+    gint    version;
     MDTUnit xyunits;
 } MDTDotsHeader;
 
 typedef struct {
     gfloat coordx;
     gfloat coordy;
-    gint forward_size;
-    gint backward_size;
+    gint   forward_size;
+    gint   backward_size;
 } MDTDotsData;
 
+typedef struct {
+    GwyContainer    *mydata;
+    GtkWidget       *view;
+    GtkWidget       *graph;
+    GtkWidget       *combobox;
+    GwySelection    *selection;
+    GwyGraphModel   *gmodel;
+    MDTMDAFrame     *dataframe;
+    MDTImageType     output;
+    gint             xc, yc;
+    gint             xres, yres;
+    gint             numpoints;
+    gint             number;
+} MDTControlsType;
+
 static gboolean       module_register       (void);
 static gint           mdt_detect            (const GwyFileDetectInfo *fileinfo,
                                              gboolean only_name);
@@ -409,24 +438,22 @@ static GwySpectra*    extract_sps_curve     (MDTScannedDataFrame *dataframe,
 static GwyDataField*  extract_mda_data      (MDTMDAFrame *dataframe);
 static GwyGraphModel* extract_mda_spectrum  (MDTMDAFrame *dataframe,
                                              guint number);
-static GwyDataField * extract_raman_image   (MDTMDAFrame *dataframe,
-                                             GwyRunType mode);
-
-static void          start_element       (GMarkupParseContext *context,
-                                          const gchar *element_name,
-                                          const gchar **attribute_names,
-                                          const gchar **attribute_values,
-                                          gpointer user_data,
-                                          GError **error);
-static void          end_element         (GMarkupParseContext *context,
-                                          const gchar *element_name,
-                                          gpointer user_data,
-                                          GError **error);
-static void          parse_text          (GMarkupParseContext *context,
-                                          const gchar *text,
-                                          gsize text_len,
-                                          gpointer user_data,
-                                          GError **error);
+static GwyBrick *     extract_raman_image   (MDTMDAFrame *dataframe);
+static void           start_element    (GMarkupParseContext *context,
+                                        const gchar *element_name,
+                                        const gchar **attribute_names,
+                                        const gchar **attribute_values,
+                                        gpointer user_data,
+                                        GError **error);
+static void           end_element      (GMarkupParseContext *context,
+                                        const gchar *element_name,
+                                        gpointer user_data,
+                                        GError **error);
+static void           parse_text       (GMarkupParseContext *context,
+                                        const gchar *text,
+                                        gsize text_len,
+                                        gpointer user_data,
+                                        GError **error);
 
 #ifdef DEBUG
 static const GwyEnum frame_types[] = {
@@ -657,7 +684,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports NT-MDT data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.15",
+    "0.16",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -720,10 +747,11 @@ mdt_load(const gchar *filename,
     gsize size;
     GError *err = NULL;
     GwyDataField *dfield = NULL;
+    GwyBrick *brick = NULL;
     GwyContainer *meta, *data = NULL;
     MDTFile mdtfile;
     GString *key;
-    guint n, i;
+    guint n, i, xres, yres;
 
     gwy_debug("");
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
@@ -752,14 +780,14 @@ mdt_load(const gchar *filename,
             if (sdframe->title) {
                 g_string_append(key, "/title");
                 gwy_container_set_string_by_name(data, key->str,
-                        g_strdup_printf("%.*s (%u)",
-                                        sdframe->title_len,
+                                        g_strdup_printf("%s (%u)",
                                         sdframe->title, i+1));
+                g_free((gpointer)sdframe->title);
             }
             else
-                gwy_app_channel_title_fall_back(data, n);
+                gwy_app_channel_title_fall_back(data, i);
 
-            meta = mdt_get_metadata(&mdtfile, n);
+            meta = mdt_get_metadata(&mdtfile, i);
             mdt_add_frame_metadata(sdframe, meta);
             g_string_printf(key, "/%d/meta", n);
             gwy_container_set_object_by_name(data, key->str, meta);
@@ -781,13 +809,18 @@ mdt_load(const gchar *filename,
                 if (mdaframe->title) {
                     g_string_append(key, "/title");
                     gwy_container_set_string_by_name(data, key->str,
-                            g_strdup_printf("%.*s (%u)",
-                                            mdaframe->title_len,
+                                            g_strdup_printf("%s (%u)",
                                             mdaframe->title, i+1));
+                    g_free((gpointer)mdaframe->title);
                 }
                 else
                     gwy_app_channel_title_fall_back(data, n);
 
+                meta = mdt_get_metadata(&mdtfile, i);
+                g_string_printf(key, "/%d/meta", n);
+                gwy_container_set_object_by_name(data, key->str, meta);
+                g_object_unref(meta);
+
                 n++;
             }
             else if ((mdaframe->nDimensions == 0 && mdaframe->nMesurands == 2)
@@ -803,21 +836,43 @@ mdt_load(const gchar *filename,
             }
             else if (mdaframe->nDimensions == 3 && mdaframe->nMesurands == 3) {
                 /* raman images */
-                dfield = extract_raman_image(mdaframe, mode);
-                g_string_printf(key, "/%d/data", n);
-                gwy_container_set_object_by_name(data, key->str, dfield);
-                g_object_unref(dfield);
-                if (mdaframe->title) {
-                    g_string_append(key, "/title");
-                    gwy_container_set_string_by_name(data, key->str,
-                            g_strdup_printf("%.*s (%u)",
-                                            mdaframe->title_len,
-                                            mdaframe->title, i+1));
-                }
-                else
-                    gwy_app_channel_title_fall_back(data, n);
+                if ((brick = extract_raman_image(mdaframe))) {
+                    g_string_printf(key, "/brick/%d", n);
+                    gwy_container_set_object_by_name(data,
+                                                     key->str,
+                                                     brick);
 
-                n++;
+                    if (mdaframe->title) {
+                        g_string_append(key, "/title");
+                        gwy_container_set_string_by_name(data, key->str,
+                                            g_strdup_printf("%s (%u)",
+                                            mdaframe->title, i+1));
+                        g_free((gpointer)mdaframe->title);
+                    }
+                    else
+                        gwy_app_channel_title_fall_back(data, n);
+
+                    xres = gwy_brick_get_xres(brick);
+                    yres = gwy_brick_get_yres(brick);
+                    dfield = gwy_data_field_new(xres, yres,
+                                                1.0, 1.0, FALSE);
+                    gwy_brick_mean_plane(brick, dfield,
+                                         0, 0, 0,
+                                         xres, yres, -1, FALSE);
+                    g_string_printf(key,"/brick/%d/preview", n);
+                    gwy_container_set_object_by_name(data,
+                                                     key->str,
+                                                     dfield);
+                    g_object_unref(brick);
+                    g_object_unref(dfield);
+
+                    meta = mdt_get_metadata(&mdtfile, i);
+                    g_string_printf(key, "/%d/meta", n);
+                    gwy_container_set_object_by_name(data, key->str, meta);
+                    g_object_unref(meta);
+
+                    n++;
+                }
             }
         }
         else if (mdtfile.frames[i].type == MDT_FRAME_SPECTROSCOPY) {
@@ -1127,7 +1182,8 @@ mdt_scanned_data_vars(const guchar *p,
         frame->title_len = gwy_get_guint32_le(&p);
         if (frame->title_len
             && (guint)(frame_size - (p - fstart)) >= frame->title_len) {
-            frame->title = p;
+            frame->title = g_convert((const gchar*)p, frame->title_len,
+                                    "UTF-8", "cp1251", NULL, NULL, NULL);
             p += frame->title_len;
             gwy_debug("title = <%.*s>", frame->title_len, frame->title);
         }
@@ -1234,9 +1290,9 @@ mdt_mda_vars(const guchar *p,
              G_GNUC_UNUSED GError **error)
 {
 
-    guint headSize, totLen, NameSize, CommSize, ViewInfoSize, SpecSize;
-    guint SourceInfoSize, VarSize, DataSize, StructLen, CellSize;
-    guint64  num;
+    guint headSize, NameSize, CommSize, ViewInfoSize, SpecSize;
+    G_GNUC_UNUSED guint totLen, SourceInfoSize, VarSize, DataSize, StructLen, CellSize;
+    G_GNUC_UNUSED guint64 num;
     const guchar *recordPointer = p;
     const guchar *structPointer;
 
@@ -1266,7 +1322,8 @@ mdt_mda_vars(const guchar *p,
     frame->title_len = NameSize;
 
     if (NameSize && (guint)(frame_size - (p - fstart)) >= NameSize) {
-        frame->title = p;
+        frame->title = g_convert((const gchar*)p, frame->title_len,
+                                 "UTF-8", "cp1251", NULL, NULL, NULL);
         p += NameSize;
     }
     else
@@ -1504,21 +1561,20 @@ extract_scanned_data(MDTScannedDataFrame *dataframe)
     return dfield;
 }
 
-static GwyGraphModel* extract_scanned_spectrum (MDTScannedDataFrame *dataframe,
-                                                guint number)
+static GwyGraphModel*
+extract_scanned_spectrum (MDTScannedDataFrame *dataframe, guint number)
 {
     GwyGraphCurveModel *spectra;
     GwyGraphModel *gmodel;
     GwySIUnit *siunitx, *siunitz;
     guint i, res;
     gdouble *xdata, *ydata;
-    gdouble xreal, zscale;
+    G_GNUC_UNUSED gdouble xreal, zscale;
     gdouble deltax;
     gint power10x, power10z;
     const gint16 *p;
     const gchar *unit;
     gchar *framename;
-
     unit = gwy_flat_enum_to_string(dataframe->x_scale.unit,
                                    G_N_ELEMENTS(mdt_units),
                                    mdt_units, mdt_units_name);
@@ -1534,10 +1590,9 @@ static GwyGraphModel* extract_scanned_spectrum (MDTScannedDataFrame *dataframe,
     siunitz = gwy_si_unit_new_parse(unit, &power10z);
     zscale = pow10(power10z)*dataframe->z_scale.step;
 
-    if (dataframe->title_len && dataframe->title)
-        framename = g_strdup_printf("%.*s (%u)",
-                                    dataframe->title_len,
-                                    dataframe->title, number);
+    if (dataframe->title_len && dataframe->title) {
+        framename = g_strdup_printf("%s (%u)", dataframe->title, number);
+    }
     else
         framename = g_strdup_printf("Unknown spectrum (%d)", number);
 
@@ -1584,9 +1639,9 @@ static GwySpectra* extract_sps_curve (MDTScannedDataFrame *dataframe,
     GwySpectra *spectra;
     GwyDataLine *dline;
     GwySIUnit *siunitx, *siunitz, *siunitcoordxy;
-    guint i, i_p, res, numpoints;
+    guint i, i_p, numpoints;
     gdouble *ydata = NULL;
-    gdouble xreal, zscale;
+    G_GNUC_UNUSED gdouble xreal, zscale;
     gdouble deltax;
     gint power10x, power10z, power10coordxy;
     const guchar *p;
@@ -1611,7 +1666,6 @@ static GwySpectra* extract_sps_curve (MDTScannedDataFrame *dataframe,
     zscale = pow10(power10z)*dataframe->z_scale.step;
 
     p = dataframe->dots;
-    res = dataframe->fm_xres;
     numpoints = dataframe->fm_ndots;
 
     /* reading coordheader */
@@ -1674,10 +1728,9 @@ static GwySpectra* extract_sps_curve (MDTScannedDataFrame *dataframe,
                     coordinates[i_p].coordy*pow10(power10coordxy));
     }
 
-    if (dataframe->title_len && dataframe->title)
-        framename = g_strdup_printf("%.*s (%u)",
-                                    dataframe->title_len,
-                                    dataframe->title, number);
+    if (dataframe->title_len && dataframe->title) {
+        framename = g_strdup_printf("%s (%u)", dataframe->title, number);
+    }
     else
         framename = g_strdup_printf("Unknown spectrum (%d)", number);
     gwy_spectra_set_title(spectra, framename);
@@ -1907,20 +1960,20 @@ extract_mda_spectrum(MDTMDAFrame *dataframe, guint number)
     MDTXMLParams params;
     GError *err = NULL;
 
-    if (dataframe->title_len && dataframe->title)
-        framename = g_strdup_printf("%.*s (%u)",
-                                    dataframe->title_len,
+    if (dataframe->title_len && dataframe->title) {
+        framename = g_strdup_printf("%s (%u)",
                                     dataframe->title, number);
+    }
     else
         framename = g_strdup_printf("Unknown spectrum (%d)", number);
 
     if (dataframe->nDimensions) {
-        xAxis = &dataframe->dimensions[0],
-              yAxis = &dataframe->mesurands[0];
+        xAxis = &dataframe->dimensions[0];
+        yAxis = &dataframe->mesurands[0];
     }
     else {
-        xAxis = &dataframe->mesurands[0],
-              yAxis = &dataframe->mesurands[1];
+        xAxis = &dataframe->mesurands[0];
+        yAxis = &dataframe->mesurands[1];
     }
 
     if (xAxis->unit && xAxis->unitLen) {
@@ -2258,175 +2311,55 @@ extract_mda_spectrum(MDTMDAFrame *dataframe, guint number)
     return gmodel;
 }
 
-static gboolean
-resize_image(GtkWidget *widget, GdkEvent *event, GtkWidget *window)
+static GwyBrick *
+extract_raman_image(MDTMDAFrame *dataframe)
 {
-    gint oldwidth, oldheight, newwidth, newheight;
-    gfloat mult;
-
-    GdkPixbuf *pixbuf = gtk_image_get_pixbuf(GTK_IMAGE(widget));
-    if (pixbuf == NULL) {
-        g_printerr("Failed to resize image\n");
-        return 1;
-    }
-
-    oldwidth  = gdk_pixbuf_get_width(pixbuf);
-    oldheight = gdk_pixbuf_get_height(pixbuf);
-    newwidth  = widget->allocation.width;
-    newheight = widget->allocation.height;
-    mult = MIN(((float) newwidth)/oldwidth,
-               ((float) newheight)/oldheight);
-
-    pixbuf = gdk_pixbuf_scale_simple(pixbuf, (int) (mult*oldwidth),
-                      (int) (mult*oldheight), GDK_INTERP_BILINEAR);
-
-    gtk_image_set_from_pixbuf(GTK_IMAGE(widget), pixbuf);
-    g_object_unref(pixbuf);
-
-    return FALSE;
-}
-
-static GwyDataField * extract_raman_image (MDTMDAFrame *dataframe,
-                                           GwyRunType mode)
-{
-    guint xsize, ysize;
+    GwyBrick *brick;
+    gint power10x, power10y, power10z, power10w;
+    GwySIUnit *siunitx, *siunity, *siunitz, *siunitw;
     const guchar *p, *px;
-    guint i, j, k;
-    gint l;
-    gdouble xspectra[1024], yspectra[1024];
-    gdouble r, g, b, x, y, z, xyzsum, wmin, wmax;
-    gdouble ymax;
-
-    GtkWidget *dialog, *image;
-    GdkPixbuf *pixbuf;
-    gint rowstride;
-    guchar *pixels;
-    guint response;
-
-    GwyDataField *dfield;
-    gdouble *data;
-    gdouble xreal, yreal, zscale;
-    gint power10xy, power10z;
-    GwySIUnit *siunitxy, *siunitz;
     const gchar *cunit;
     gchar *unit;
+    gint xres, yres, zres;
+    gint i, j, k;
+    gdouble xreal, yreal, zscale, wscale, w;
+    gdouble *data;
+    GwyDataLine *cal;
+    MDTMDACalibration *xAxis, *yAxis, *zAxis, *wAxis;
 
-    /* CIE 2deg xyz functions for standart observer (360-780 nm)   */
-    /* from http://www.cis.rit.edu/mcsl/online/CIE/StdObsFuncs.htm */
-    static gdouble cie_2deg_xyz[85][3] = {
-    { 0.000130, 0.000004, 0.000606 },
-    { 0.000232, 0.000007, 0.001086 },
-    { 0.000415, 0.000012, 0.001946 },
-    { 0.000742, 0.000022, 0.003486 },
-    { 0.001368, 0.000039, 0.006450 },
-    { 0.002236, 0.000064, 0.010550 },
-    { 0.004243, 0.000120, 0.020050 },
-    { 0.007650, 0.000217, 0.036210 },
-    { 0.014310, 0.000396, 0.067850 },
-    { 0.023190, 0.000640, 0.110200 },
-    { 0.043510, 0.001210, 0.207400 },
-    { 0.077630, 0.002180, 0.371300 },
-    { 0.134380, 0.004000, 0.645600 },
-    { 0.214770, 0.007300, 1.039050 },
-    { 0.283900, 0.011600, 1.385600 },
-    { 0.328500, 0.016840, 1.622960 },
-    { 0.348280, 0.023000, 1.747060 },
-    { 0.348060, 0.029800, 1.782600 },
-    { 0.336200, 0.038000, 1.772110 },
-    { 0.318700, 0.048000, 1.744100 },
-    { 0.290800, 0.060000, 1.669200 },
-    { 0.251100, 0.073900, 1.528100 },
-    { 0.195360, 0.090980, 1.287640 },
-    { 0.142100, 0.112600, 1.041900 },
-    { 0.095640, 0.139020, 0.812950 },
-    { 0.057950, 0.169300, 0.616200 },
-    { 0.032010, 0.208020, 0.465180 },
-    { 0.014700, 0.258600, 0.353300 },
-    { 0.004900, 0.323000, 0.272000 },
-    { 0.002400, 0.407300, 0.212300 },
-    { 0.009300, 0.503000, 0.158200 },
-    { 0.029100, 0.608200, 0.111700 },
-    { 0.063270, 0.710000, 0.078250 },
-    { 0.109600, 0.793200, 0.057250 },
-    { 0.165500, 0.862000, 0.042160 },
-    { 0.225750, 0.914850, 0.029840 },
-    { 0.290400, 0.954000, 0.020300 },
-    { 0.359700, 0.980300, 0.013400 },
-    { 0.433450, 0.994950, 0.008750 },
-    { 0.512050, 1.000000, 0.005750 },
-    { 0.594500, 0.995000, 0.003900 },
-    { 0.678400, 0.978600, 0.002750 },
-    { 0.762100, 0.952000, 0.002100 },
-    { 0.842500, 0.915400, 0.001800 },
-    { 0.916300, 0.870000, 0.001650 },
-    { 0.978600, 0.816300, 0.001400 },
-    { 1.026300, 0.757000, 0.001100 },
-    { 1.056700, 0.694900, 0.001000 },
-    { 1.062200, 0.631000, 0.000800 },
-    { 1.045600, 0.566800, 0.000600 },
-    { 1.002600, 0.503000, 0.000340 },
-    { 0.938400, 0.441200, 0.000240 },
-    { 0.854450, 0.381000, 0.000190 },
-    { 0.751400, 0.321000, 0.000100 },
-    { 0.642400, 0.265000, 0.000050 },
-    { 0.541900, 0.217000, 0.000030 },
-    { 0.447900, 0.175000, 0.000020 },
-    { 0.360800, 0.138200, 0.000010 },
-    { 0.283500, 0.107000, 0.000000 },
-    { 0.218700, 0.081600, 0.000000 },
-    { 0.164900, 0.061000, 0.000000 },
-    { 0.121200, 0.044580, 0.000000 },
-    { 0.087400, 0.032000, 0.000000 },
-    { 0.063600, 0.023200, 0.000000 },
-    { 0.046770, 0.017000, 0.000000 },
-    { 0.032900, 0.011920, 0.000000 },
-    { 0.022700, 0.008210, 0.000000 },
-    { 0.015840, 0.005723, 0.000000 },
-    { 0.011359, 0.004102, 0.000000 },
-    { 0.008111, 0.002929, 0.000000 },
-    { 0.005790, 0.002091, 0.000000 },
-    { 0.004109, 0.001484, 0.000000 },
-    { 0.002899, 0.001047, 0.000000 },
-    { 0.002049, 0.000740, 0.000000 },
-    { 0.001440, 0.000520, 0.000000 },
-    { 0.001000, 0.000361, 0.000000 },
-    { 0.000690, 0.000249, 0.000000 },
-    { 0.000476, 0.000172, 0.000000 },
-    { 0.000332, 0.000120, 0.000000 },
-    { 0.000235, 0.000085, 0.000000 },
-    { 0.000166, 0.000060, 0.000000 },
-    { 0.000117, 0.000042, 0.000000 },
-    { 0.000083, 0.000030, 0.000000 },
-    { 0.000059, 0.000021, 0.000000 },
-    { 0.000042, 0.000015, 0.000000 },
-    };
-
-    gdouble xr = 3.240479;
-    gdouble xg = -0.969256;
-    gdouble xb = 0.055648;
-    gdouble yr = -1.53715;
-    gdouble yg = 1.875992;
-    gdouble yb = -0.204043;
-    gdouble zr = -0.498535;
-    gdouble zg = 0.041556;
-    gdouble zb = 1.057311;
-
-    MDTMDACalibration *xAxis = &dataframe->dimensions[0],
-                      *yAxis = &dataframe->dimensions[1],
-                      *zAxis = &dataframe->mesurands[0];
+    xAxis = &dataframe->dimensions[0];
+    yAxis = &dataframe->dimensions[1];
+    zAxis = &dataframe->mesurands[1];
+    wAxis = &dataframe->mesurands[0];
+    xres  = (xAxis->maxIndex - xAxis->minIndex + 1);
+    yres  = (yAxis->maxIndex - yAxis->minIndex + 1);
+    zres  = 1024;
 
     if (xAxis->unit && xAxis->unitLen) {
         unit = g_strndup(xAxis->unit, xAxis->unitLen);
-        siunitxy = gwy_si_unit_new_parse(unit, &power10xy);
+        siunitx = gwy_si_unit_new_parse(unit, &power10x);
         g_free(unit);
     }
     else {
         cunit = gwy_flat_enum_to_string(unitCodeForSiCode(xAxis->siUnit),
                                         G_N_ELEMENTS(mdt_units),
                                         mdt_units, mdt_units_name);
-        siunitxy = gwy_si_unit_new_parse(cunit, &power10xy);
+        siunitx = gwy_si_unit_new_parse(cunit, &power10x);
     }
-    gwy_debug("xy unit power %d", power10xy);
+    gwy_debug("x unit power %d", power10x);
+
+    if (yAxis->unit && yAxis->unitLen) {
+        unit = g_strndup(yAxis->unit, yAxis->unitLen);
+        siunity = gwy_si_unit_new_parse(unit, &power10y);
+        g_free(unit);
+    }
+    else {
+        cunit = gwy_flat_enum_to_string(unitCodeForSiCode(yAxis->siUnit),
+                                        G_N_ELEMENTS(mdt_units),
+                                        mdt_units, mdt_units_name);
+        siunity = gwy_si_unit_new_parse(cunit, &power10y);
+    }
+    gwy_debug("y unit power %d", power10y);
 
     if (zAxis->unit && zAxis->unitLen) {
         unit = g_strndup(zAxis->unit, zAxis->unitLen);
@@ -2439,136 +2372,69 @@ static GwyDataField * extract_raman_image (MDTMDAFrame *dataframe,
                                         mdt_units, mdt_units_name);
         siunitz = gwy_si_unit_new_parse(cunit, &power10z);
     }
-    gwy_debug("z unit power %d", power10xy);
+    gwy_debug("z unit power %d", power10z);
 
-    xreal = pow10(power10xy) * xAxis->scale;
-    yreal = pow10(power10xy) * yAxis->scale;
-    zscale = pow10(power10z) * zAxis->scale;
-
-    xsize = (dataframe->dimensions[0].maxIndex
-           - dataframe->dimensions[0].minIndex + 1);
-    ysize = (dataframe->dimensions[1].maxIndex
-           - dataframe->dimensions[1].minIndex + 1);
-
-    /* FIXME: real size is wrong */
-    dfield = gwy_data_field_new(xsize, ysize,
-                                xreal*(xsize - 1), yreal*(ysize - 1), FALSE);
-
-    gwy_data_field_set_si_unit_xy(dfield, siunitxy);
-    g_object_unref(siunitxy);
-    gwy_data_field_set_si_unit_z(dfield, siunitz);
-    g_object_unref(siunitz);
-
-    data = gwy_data_field_get_data(dfield);
-
-    if (mode == GWY_RUN_INTERACTIVE) {
-        dialog = gtk_dialog_new_with_buttons(_("Raman Image"), NULL, 0,
-                                 GTK_STOCK_OK, GTK_RESPONSE_OK,
-                                 NULL);
-        gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
-        gtk_dialog_set_default_response(GTK_DIALOG(dialog),
-                                        GTK_RESPONSE_OK);
+    if (wAxis->unit && wAxis->unitLen) {
+        unit = g_strndup(wAxis->unit, wAxis->unitLen);
+        siunitw = gwy_si_unit_new_parse(unit, &power10w);
+        g_free(unit);
     }
+    else {
+        cunit = gwy_flat_enum_to_string(unitCodeForSiCode(wAxis->siUnit),
+                                        G_N_ELEMENTS(mdt_units),
+                                        mdt_units, mdt_units_name);
+        siunitw = gwy_si_unit_new_parse(cunit, &power10w);
+    }
+    gwy_debug("w unit power %d", power10w);
 
-    image = gtk_image_new();
+    xreal = pow10(power10x) * xAxis->scale;
+    yreal = pow10(power10y) * yAxis->scale;
+    zscale = pow10(power10z) * zAxis->scale;
+    wscale = pow10(power10w) * wAxis->scale;
 
-    pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8,
-                                xsize, ysize);
-    gdk_pixbuf_fill(pixbuf, 0);
+    brick = gwy_brick_new(xres, yres, zres,
+                          xreal * xres,
+                          yreal * yres,
+                          zscale * zres,
+                          TRUE);
 
-    pixels = gdk_pixbuf_get_pixels(pixbuf);
-    rowstride = gdk_pixbuf_get_rowstride(pixbuf);
+    data = gwy_brick_get_data(brick);
 
-    ymax = 0.0;
+    for (k = 0; k < zres; k++) {
+        p = (guchar *)dataframe->image;
+        p += k * sizeof(gfloat);
+        for (i = 0; i < yres; i++)
+            for (j = 0; j < xres; j++) {
+                w = (gdouble)gwy_get_gfloat_le(&p);
+                *(data++) = w * wscale;
+                p += (zres - 1) * sizeof(gfloat);
+
+            }
+    }
 
     p = (guchar *)dataframe->image;
-    px = p + 1024*xsize*ysize*sizeof(gfloat);
-    for (k = 0; k < 1024; k++) {
-            xspectra[k] = (gdouble)gwy_get_gfloat_le(&px);
-        }
-
-    for (i = 0; i < ysize; i++) {
-        for (j = 0; j < xsize; j++) {
-            x = 0;
-            y = 0;
-            z = 0;
-            for (k = 0; k < 1024; k++) {
-                yspectra[k] = (gdouble)gwy_get_gfloat_le(&p);
-                if (yspectra[k] > 1e-9) /* not a black pixel */
-                    yspectra[k] -= 550.0; /* background noise */
-                l = (gint)((xspectra[k] - 360.0)/5);
-                if ((l >= 0) && (l < 85) && (k < 1023)) {
-                    x += yspectra[k]*cie_2deg_xyz[l][0]
-                        *(xspectra[k+1]-xspectra[k]);
-                    y += yspectra[k]*cie_2deg_xyz[l][1]
-                        *(xspectra[k+1]-xspectra[k]);
-                    z += yspectra[k]*cie_2deg_xyz[l][2]
-                        *(xspectra[k+1]-xspectra[k]);
-                }
-            }
-
-            x /= 1024.0;
-            y /= 1024.0;
-            z /= 1024.0;
-            ymax = (y > ymax) ? y : ymax;
-            *(data++) = y;
-            xyzsum = x + y + z;
-            x /= xyzsum;
-            y /= xyzsum;
-            z /= xyzsum;
-            r = xr*x + yr*y + zr*z;
-            g = xg*x + yg*y + zg*z;
-            b = xb*x + yb*y + zb*z;
-            wmin = (r < g) ? r : g;
-            wmin = (wmin < b) ? wmin : b;
-            if (wmin < 0.0) {
-                wmin = -wmin;
-                r += wmin;
-                g += wmin;
-                b += wmin;
-            }
-            wmax = (r > g) ? r : g;
-            wmax = (wmax > b) ? wmax : b;
-            if (wmax > 1.0) {
-                r /= wmax;
-                g /= wmax;
-                b /= wmax;
-            }
-
-            pixels[i*rowstride+3*j]   = (guchar)(r*255.0);
-            pixels[i*rowstride+3*j+1] = (guchar)(g*255.0);
-            pixels[i*rowstride+3*j+2] = (guchar)(b*255.0);
-        }
+    px = p + xres * yres * zres * sizeof(gfloat);
+    cal = gwy_data_line_new(zres, zres, FALSE);
+    data = gwy_data_line_get_data(cal);
+    for (k = 0; k < zres; k++) {
+        *(data++) = zscale * (gdouble)gwy_get_gfloat_le(&px);
     }
+    gwy_data_line_set_si_unit_y(cal, siunitz);
+    gwy_brick_set_zcalibration(brick, cal);
 
-    data = gwy_data_field_get_data(dfield);
-    for (i = 0; i < ysize; i++) {
-        for (j = 0; j < xsize; j++) {
-            y = *(data++)/ymax;
-            pixels[i*rowstride+3*j] *= y;
-            pixels[i*rowstride+3*j+1] *= y;
-            pixels[i*rowstride+3*j+2] *= y;
-        }
-    }
-    gtk_image_set_from_pixbuf(GTK_IMAGE(image), pixbuf);
-    g_object_unref(pixbuf);
+    g_object_unref(cal);
 
-    /* FIXME: this is temporary code to show raman images, need rework */
-    if (mode == GWY_RUN_INTERACTIVE) {
-        g_signal_connect(image, "expose-event", G_CALLBACK(resize_image),
-                        (gpointer)dialog);
-        gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), image,
-                           TRUE, TRUE, 0);
-        gtk_widget_show_all(dialog);
-        do {
-            response = gtk_dialog_run(GTK_DIALOG(dialog));
-        } while ((response != GTK_RESPONSE_OK)
-              && (response != GTK_RESPONSE_DELETE_EVENT));
+    gwy_brick_set_si_unit_x(brick, siunitx);
+    gwy_brick_set_si_unit_y(brick, siunity);
+    gwy_brick_set_si_unit_z(brick, siunitz);
+    gwy_brick_set_si_unit_w(brick, siunitw);
 
-        gtk_widget_destroy(dialog);
-    }
+    g_object_unref(siunitx);
+    g_object_unref(siunity);
+    g_object_unref(siunitz);
+    g_object_unref(siunitw);
 
-    return dfield;
+    return brick;
 }
 
 static void
