@@ -1,5 +1,5 @@
 /*
- *  $Id: omicron.c 12492 2011-05-30 20:59:42Z yeti-dn $
+ *  $Id: omicron.c 15039 2013-05-14 11:36:04Z yeti-dn $
  *  Copyright (C) 2006-2011 David Necas (Yeti), Petr Klapetek, Markus Pristovsek
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net,
  *  prissi@gift.physik.tu-berlin.de.
@@ -16,7 +16,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 /**
@@ -24,13 +24,24 @@
  * <mime-type type="application/x-omicron-spm">
  *   <comment>Omicron SCALA data parameters</comment>
  *   <magic priority="80">
- *     <match type="string" offset="0:160" value="Omicron SPM Control"/>
+ *     <match type="string" offset="0" value=";">
+ *       <match type="string" offset="0:160" value="Omicron SPM Control"/>
+ *     </match>
  *   </magic>
  *   <glob pattern="*.par"/>
  *   <glob pattern="*.PAR"/>
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-FILEMAGIC]
+ * # Old Omicron SCALA
+ * # Usually accompanied by an unidentifiable data file.
+ * # Must search for string, it starts with some ;-commented lines.
+ * 0 byte 0x3b
+ * >&0 search/160 Omicron\ SPM\ Control Omicron SCALA SPM parameters text header
+ **/
+
 /**
  * [FILE-MAGIC-USERGUIDE]
  * Omicron SCALA
@@ -52,7 +63,7 @@
 
 #include "err.h"
 
-#define MAGIC "Omicron SPM Control"
+#define MAGIC "Parameter file for SPM data."
 #define MAGIC_SIZE (sizeof(MAGIC)-1)
 
 #define EXTENSION_HEADER ".par"
@@ -148,7 +159,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports Omicron data files (two-part .par + .tf*, .tb*, .sf*, .sb*)."),
     "Yeti <yeti@gwyddion.net>",
-    "0.8",
+    "0.10",
     "David Nečas (Yeti) & Petr Klapetek & Markus Pristovsek",
     "2006",
 };
@@ -172,22 +183,24 @@ static gint
 omicron_detect(const GwyFileDetectInfo *fileinfo,
                gboolean only_name)
 {
-    guint i;
+    const gchar *p = fileinfo->head;
 
     if (only_name)
         return g_str_has_suffix(fileinfo->name_lowercase, EXTENSION_HEADER)
                ? 15 : 0;
 
     /* Quick check to skip most non-matching files */
-    if (fileinfo->buffer_len < 100
-        || fileinfo->head[0] != ';')
+    if (fileinfo->buffer_len < 100)
         return 0;
 
-    for (i = 1; i + MAGIC_SIZE+1 < fileinfo->buffer_len; i++) {
-        if (fileinfo->head[i] != ';' && !g_ascii_isspace(fileinfo->head[i]))
-            break;
-    }
-    if (memcmp(fileinfo->head + i, MAGIC, MAGIC_SIZE) == 0)
+    p = fileinfo->head;
+    if (*p != ';' || !(p = strchr(p+1, ';')) || !(p = strchr(p+1, ';')))
+        return 0;
+
+    do {
+        p++;
+    } while (g_ascii_isspace(*p));
+    if (memcmp(p, MAGIC, MAGIC_SIZE) == 0)
         return 100;
 
     return 0;
@@ -855,10 +868,10 @@ omicron_read_cs_data(OmicronFile *ofile,
             i = 0;
         }
         if (strstr(line, "BEGIN") && !strstr(line, "COORD")) {
+            gdouble real = ((channel->end - channel->start)*channel->npoints
+                            /(channel->npoints - 1));
             /* Read spectroscopy points */
-            dline = gwy_data_line_new(channel->npoints,
-                                      channel->end - channel->start,
-                                      FALSE);
+            dline = gwy_data_line_new(channel->npoints, real, FALSE);
             gwy_data_line_set_offset(dline, (channel->start));
             data = gwy_data_line_get_data(dline);
             j = 0;
@@ -1014,9 +1027,9 @@ omicron_read_be_data(OmicronFile *ofile,
     g_object_unref(coord_unit);
 
     for (i = 0; i < gyres; i++) {
-        gdouble y = ofile->yreal*gystep*(ofile->yres-1 - i)/gyres;
+        gdouble y = ofile->yreal*gystep*(gyres-1 - i)/ofile->yres;
         for (j = 0; j < gxres; j++) {
-            gdouble x = ofile->xreal*gxstep*j/gxres;
+            gdouble x = ofile->xreal*gxstep*j/ofile->xres;
             GwyDataLine *dline = gwy_data_line_new(channel->npoints, xreal,
                                                    FALSE);
 
