@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: rawfile.c 12380 2011-05-09 22:13:30Z yeti-dn $
+ *  @(#) $Id: rawfile.c 13809 2012-09-19 18:19:21Z yeti-dn $
  *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 /**
@@ -399,7 +399,6 @@ rawfile_dialog(RawFileArgs *args,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          GTK_STOCK_OK, GTK_RESPONSE_OK,
                                          NULL);
-    gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
     controls.dialog = dialog;
     controls.args = args;
@@ -964,22 +963,16 @@ rawfile_dialog_preset_page(RawFileArgs *args,
     GtkCellRenderer *renderer;
     GtkTreeViewColumn *column;
     GtkTreeIter iter;
-    GtkWidget *vbox, *label, *table, *button, *scroll, *bbox;
+    GtkWidget *vbox, *table, *button, *scroll, *bbox;
     guint i, row;
 
-    row = 0;
     vbox = gtk_vbox_new(FALSE, 0);   /* to prevent notebook expanding tables */
     gtk_container_set_border_width(GTK_CONTAINER(vbox), 6);
 
-    label = gtk_label_new_with_mnemonic(_("Preset l_ist"));
-    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
-    gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
-
     store = gwy_inventory_store_new(gwy_raw_file_presets());
     controls->presetlist = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
     gtk_tree_view_set_rules_hint(GTK_TREE_VIEW(controls->presetlist), TRUE);
     g_object_unref(store);
-    gtk_label_set_mnemonic_widget(GTK_LABEL(label), controls->presetlist);
 
     for (i = 0; i < G_N_ELEMENTS(columns); i++) {
         renderer = gtk_cell_renderer_text_new();
@@ -1030,6 +1023,7 @@ rawfile_dialog_preset_page(RawFileArgs *args,
                              G_CALLBACK(preset_delete_cb), controls);
 
     table = gtk_table_new(1, 3, FALSE);
+    gtk_table_set_col_spacings(GTK_TABLE(table), 6);
     gtk_box_pack_start(GTK_BOX(vbox), table, FALSE, FALSE, 4);
     row = 0;
 
