@@ -1,5 +1,5 @@
 /*
- *  $Id: rawxyz.c 12379 2011-05-09 22:11:33Z yeti-dn $
+ *  $Id: rawxyz.c 14929 2013-04-20 18:09:35Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 /**
@@ -222,7 +222,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports raw XYZ data files."),
     "Yeti <yeti@gwyddion.net>",
-    "1.0",
+    "1.2",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -387,7 +387,6 @@ rawxyz_dialog(RawXYZArgs *args,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          GTK_STOCK_OK, GTK_RESPONSE_OK,
                                          NULL);
-    gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
     controls.dialog = dialog;
 
@@ -931,7 +930,6 @@ preview(RawXYZControls *controls)
     GwyDataField *dfield;
     GdkPixbuf *pixbuf;
     GtkWidget *entry;
-    gdouble avg, rms;
     gint xres, yres;
     GError *error = NULL;
 
@@ -945,16 +943,15 @@ preview(RawXYZControls *controls)
     args->yres = PREVIEW_SIZE*yres/MAX(xres, yres);
     dfield = rawxyz_do(controls->rfile, args, &error);
     /* Regular grids are always created at full size. */
-    gwy_data_field_resample(dfield, args->xres, args->yres,
-                            GWY_INTERPOLATION_KEY);
+    if (dfield)
+        gwy_data_field_resample(dfield, args->xres, args->yres,
+                                GWY_INTERPOLATION_KEY);
     pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, FALSE, 8,
                             args->xres, args->yres);
     args->xres = xres;
     args->yres = yres;
     if (dfield) {
         triangulation_info(controls);
-        avg = gwy_data_field_get_avg(dfield);
-        rms = gwy_data_field_get_rms(dfield);
         gwy_pixbuf_draw_data_field(pixbuf, dfield, controls->gradient);
         g_object_unref(dfield);
     }
@@ -1670,8 +1667,8 @@ check_regular_grid(RawXYZFile *rfile)
                          rfile->points->len-1);
     xstep = rfile->xstep = (pt2->x - pt1->x)/(xres - 1);
     ystep = rfile->ystep = (pt2->y - pt1->y)/(yres - 1);
-    xeps = 0.001*fabs(xstep);
-    yeps = 0.001*fabs(ystep);
+    xeps = 0.05*fabs(xstep);
+    yeps = 0.05*fabs(ystep);
 
     if (rfile->regular == RAW_XYZ_REGULAR_X) {
         for (i = 0; i < yres; i++) {
@@ -1703,6 +1700,8 @@ check_regular_grid(RawXYZFile *rfile)
     return TRUE;
 }
 
+static const gchar xres_key[]          = "/module/rawxyz/xres";
+static const gchar yres_key[]          = "/module/rawxyz/yres";
 static const gchar exterior_key[]      = "/module/rawxyz/exterior";
 static const gchar interpolation_key[] = "/module/rawxyz/interpolation";
 static const gchar xy_units_key[]      = "/module/rawxyz/xy-units";
@@ -1717,6 +1716,8 @@ rawxyz_sanitize_args(RawXYZArgs *args)
     if (args->exterior != GWY_EXTERIOR_MIRROR_EXTEND
         && args->exterior != GWY_EXTERIOR_PERIODIC)
         args->exterior = GWY_EXTERIOR_BORDER_EXTEND;
+    args->xres = CLAMP(args->xres, 2, 16384);
+    args->yres = CLAMP(args->yres, 2, 16384);
 }
 
 static void
@@ -1732,6 +1733,8 @@ rawxyz_load_args(GwyContainer *container,
                                      (const guchar**)&args->xy_units);
     gwy_container_gis_string_by_name(container, z_units_key,
                                      (const guchar**)&args->z_units);
+    gwy_container_gis_int32_by_name(container, xres_key, &args->xres);
+    gwy_container_gis_int32_by_name(container, yres_key, &args->yres);
 
     rawxyz_sanitize_args(args);
     args->xy_units = g_strdup(args->xy_units ? args->xy_units : "");
@@ -1749,6 +1752,8 @@ rawxyz_save_args(GwyContainer *container,
                                      g_strdup(args->xy_units));
     gwy_container_set_string_by_name(container, z_units_key,
                                      g_strdup(args->z_units));
+    gwy_container_set_int32_by_name(container, xres_key, args->xres);
+    gwy_container_set_int32_by_name(container, yres_key, args->yres);
 }
 
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
