@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include "config.h"
@@ -108,14 +108,10 @@ module_register(void)
 static void
 cut(GwyGraph *graph)
 {
-    GwyContainer *settings;
     CutArgs args;
 
     memset(&args, 0, sizeof(CutArgs));
-
     args.parent_graph = graph;
-
-    settings = gwy_app_settings_get();
     cut_dialog(&args);
 }
 
@@ -145,7 +141,6 @@ cut_dialog(CutArgs *args)
 
     dialog = gtk_dialog_new_with_buttons(_("Cut Graph"), NULL, 0, NULL);
     controls.dialog = dialog;
-    gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
     gtk_dialog_add_button(GTK_DIALOG(dialog),
                           GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
     gtk_dialog_add_button(GTK_DIALOG(dialog),
