@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: fft_filter_1d.c 11027 2010-01-27 08:50:00Z yeti-dn $
+ *  @(#) $Id: fft_filter_1d.c 13809 2012-09-19 18:19:21Z yeti-dn $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include "config.h"
@@ -126,7 +126,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("FFT filtering"),
     "Petr Klapetek <petr@klapetek.cz>",
-    "2.4",
+    "2.5",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -204,7 +204,6 @@ fftf_1d_dialog(Fftf1dArgs *args,
                           GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
     gtk_dialog_add_button(GTK_DIALOG(dialog),
                           GTK_STOCK_OK, GTK_RESPONSE_OK);
-    gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
 
     hbox = gtk_hbox_new(FALSE, 4);
@@ -431,6 +430,11 @@ restore_ps(Fftf1dControls *controls, Fftf1dArgs *args)
                  "mode", GWY_GRAPH_CURVE_LINE,
                  "description", "Fourier Modulus Density",
                   NULL);
+    g_object_set(controls->gmodel,
+                 "si-unit-x", gwy_data_line_get_si_unit_x(dline),
+                 "axis-label-bottom", "k",
+                 "axis-label-left", "",
+                 NULL);
 
     gwy_graph_model_add_curve(controls->gmodel, cmodel);
 
