@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: fft_profile.c 11330 2010-06-24 08:13:31Z yeti-dn $
+ *  @(#) $Id: fft_profile.c 13809 2012-09-19 18:19:21Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include "config.h"
@@ -192,7 +192,6 @@ prof_dialog(ProfArgs *args,
                                                   GTK_RESPONSE_OK,
                                                   NULL);
     dialog = GTK_DIALOG(controls.dialog);
-    gtk_dialog_set_has_separator(dialog, FALSE);
     gtk_dialog_set_default_response(dialog, GTK_RESPONSE_OK);
     gtk_dialog_set_response_sensitive(dialog, GTK_RESPONSE_OK, FALSE);
 
