@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: merge.c 12393 2011-05-11 07:57:09Z yeti-dn $
+ *  @(#) $Id: merge.c 14654 2013-02-17 15:01:22Z dn2010 $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include "config.h"
@@ -89,6 +89,7 @@ static gboolean merge_data_filter    (GwyContainer *data,
                                       gint id,
                                       gpointer user_data);
 static gboolean merge_do             (MergeArgs *args);
+static void     merge_do_uncorrelated(MergeArgs *args);
 static void     merge_direction_cb   (GtkWidget *combo,
                                       MergeArgs *args);
 static void     merge_mode_cb        (GtkWidget *combo,
@@ -154,7 +155,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Merges two images."),
     "Petr Klapetek <klapetek@gwyddion.net>",
-    "1.3",
+    "1.5",
     "David Nečas (Yeti) & Petr Klapetek",
     "2006",
 };
@@ -190,8 +191,12 @@ merge(GwyContainer *data, GwyRunType run)
     gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD_ID, &args.op1.id, 0);
     args.op2 = args.op1;
 
-    if (merge_dialog(&args))
-        merge_do(&args);
+    if (merge_dialog(&args)) {
+        if (args.mode == GWY_MERGE_MODE_NONE)
+            merge_do_uncorrelated(&args);
+        else
+            merge_do(&args);
+    }
 
     merge_save_args(settings, &args);
 }
@@ -207,7 +212,6 @@ merge_dialog(MergeArgs *args)
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          GTK_STOCK_OK, GTK_RESPONSE_OK,
                                          NULL);
-    gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
 
     table = gtk_table_new(4, 4, FALSE);
@@ -346,8 +350,8 @@ merge_do(MergeArgs *args)
     gint xshift, yshift;
     GQuark quark;
     gint newid;
-    GwyMergeDirectionType real_dir;
-    GwyMergeBoundaryType real_boundary;
+    GwyMergeDirectionType real_dir = args->direction;
+    GwyMergeBoundaryType real_boundary = args->boundary;
     gint px1, py1, px2, py2;
 
     quark = gwy_app_get_data_key_for_id(args->op1.id);
@@ -371,13 +375,9 @@ merge_do(MergeArgs *args)
 
         if (args->boundary == GWY_MERGE_BOUNDARY_FIRST)
             real_boundary = GWY_MERGE_BOUNDARY_SECOND;
-        if (args->boundary == GWY_MERGE_BOUNDARY_SECOND)
+        else if (args->boundary == GWY_MERGE_BOUNDARY_SECOND)
             real_boundary = GWY_MERGE_BOUNDARY_FIRST;
     }
-    else {
-        real_dir = args->direction;
-        real_boundary = args->boundary;
-    }
 
     result = gwy_data_field_new_alike(dfield1, FALSE);
 
@@ -392,18 +392,18 @@ merge_do(MergeArgs *args)
         cdata.x = 0;
         cdata.y = 0;
         cdata.width = xres1;
-        cdata.height = yres1/3;
+        cdata.height = yres1/2;
         kdata.width = MIN(xres2, cdata.width/2);
         kdata.height = MIN(yres2, cdata.height/3);
         kdata.x = MAX(0, xres2/2 - kdata.width/2);
-        kdata.y = MAX(0, yres2  - cdata.height/2);
+        kdata.y = MAX(0, yres2 - cdata.height/3);
         break;
 
         case GWY_MERGE_DIRECTION_DOWN:
         cdata.x = 0;
-        cdata.y = yres1 - (yres1/3);
+        cdata.y = yres1 - (yres1/2);
         cdata.width = xres1;
-        cdata.height = yres1/3;
+        cdata.height = yres1/2;
         kdata.width = MIN(xres2, cdata.width/2);
         kdata.height = MIN(yres2, cdata.height/3);
         kdata.x = MAX(0, xres2/2 - kdata.width/2);
@@ -411,9 +411,9 @@ merge_do(MergeArgs *args)
         break;
 
         case GWY_MERGE_DIRECTION_RIGHT:
-        cdata.x = xres1 - (xres1/3);
+        cdata.x = xres1 - (xres1/2);
         cdata.y = 0;
-        cdata.width = xres1/3;
+        cdata.width = xres1/2;
         cdata.height = yres1;
         kdata.width = MIN(xres2, cdata.width/3);
         kdata.height = MIN(yres2, cdata.height/2);
@@ -424,7 +424,7 @@ merge_do(MergeArgs *args)
         case GWY_MERGE_DIRECTION_LEFT:
         cdata.x = 0;
         cdata.y = 0;
-        cdata.width = xres1/3;
+        cdata.width = xres1/2;
         cdata.height = yres1;
         kdata.width = MIN(xres2, cdata.width/3);
         kdata.height = MIN(yres2, cdata.height/2);
@@ -460,17 +460,16 @@ merge_do(MergeArgs *args)
     }
 
     find_score_maximum(correlation_score, &max_col, &max_row);
-    gwy_debug("c: %d %d %dx%d  k: %d %d %dx%d res: %d %d\n",
-           cdata.x,
-           cdata.y,
-           cdata.width,
-           cdata.height,
-           kdata.x,
-           kdata.y,
-           kdata.width,
-           kdata.height,
-           max_col, max_row
-            );
+    gwy_debug("c: %d %d %dx%d  k: %d %d %dx%d res: %d %d",
+              cdata.x,
+              cdata.y,
+              cdata.width,
+              cdata.height,
+              kdata.x,
+              kdata.y,
+              kdata.width,
+              kdata.height,
+              max_col, max_row);
     /* enlarge result field to fit the new data */
     switch (real_dir) {
         case GWY_MERGE_DIRECTION_UP:
@@ -480,23 +479,41 @@ merge_do(MergeArgs *args)
                       (MIN(0, (max_col - cdata.width/2)) - xres2/2));
         newyres = yres1 + yres2 - max_row - kdata.height/2;
 
+        px1 = xshift - max_col + kdata.width/2 + kdata.x;
+        py1 = yres2 - max_row - kdata.height/2;
         px2 = xshift;
         py2 = 0;
-        px1 = xshift - ((max_col - kdata.width/2) - kdata.x);
-        py1 = yres2 - max_row - kdata.height/2;
         break;
 
         case GWY_MERGE_DIRECTION_DOWN:
-        xshift = MAX(0,  -(max_col - cdata.width/2) - (xres1 - xres2)/2);
+        xshift = MAX(0,  (max_col - cdata.width/2) + (xres1 - xres2)/2);
         newxres =  MAX(MAX(xres1, xres2),
                       (MAX(0, (max_col - cdata.width/2)) + xres2/2) -
                       (MIN(0, (max_col - cdata.width/2)) - xres2/2));
         newyres = cdata.y + (max_row - kdata.height/2) + yres2;;
 
-        px1 = xshift;
+        px1 = xshift - max_col + kdata.width/2 + kdata.x;
         py1 = 0;
-        px2 = xshift + ((max_col - kdata.width/2) - kdata.x);
-        py2 = cdata.y + (max_row - kdata.height/2);
+        px2 = xshift;
+        py2 = cdata.y + max_row - kdata.height/2;
+        break;
+
+        case GWY_MERGE_DIRECTION_RIGHT:
+        yshift = MAX(0,  (max_row - cdata.height/2) + (yres1 - yres2)/2);
+        newxres = cdata.x + (max_col - kdata.width/2) + xres2;
+        newyres = MAX(MAX(yres1, yres2),
+                      (MAX(0, (max_row - cdata.height/2)) + yres2/2) -
+                      (MIN(0, (max_row - cdata.height/2)) - yres2/2));
+        gwy_debug("%d %d %d %d",
+                  yres1, yres2,
+                  (MAX(0, (max_row - cdata.height/2)) + yres2/2),
+                  (MIN(0, (max_row - cdata.height/2)) - yres2/2));
+        gwy_debug("newyres: %d, yshift: %d", newyres, yshift);
+
+        px1 = 0;
+        py1 = yshift - max_row + kdata.height/2 + kdata.y;
+        px2 = cdata.x + max_col - kdata.width/2;;
+        py2 = yshift;
         break;
 
         case GWY_MERGE_DIRECTION_LEFT:
@@ -505,35 +522,16 @@ merge_do(MergeArgs *args)
         newyres = MAX(MAX(yres1, yres2),
                       (MAX(0, (max_row - cdata.height/2)) + yres2/2) -
                       (MIN(0, (max_row - cdata.height/2)) - yres2/2));
-        gwy_debug("%d %d %d %d\n",
-               yres1, yres2,
-               (MAX(0, (max_row - cdata.height/2)) + yres2/2),
-               (MIN(0, (max_row - cdata.height/2)) - yres2/2));
-        gwy_debug("newyres: %d, yshift: %d\n", newyres, yshift);
+        gwy_debug("%d %d %d %d",
+                  yres1, yres2,
+                  (MAX(0, (max_row - cdata.height/2)) + yres2/2),
+                  (MIN(0, (max_row - cdata.height/2)) - yres2/2));
+        gwy_debug("newyres: %d, yshift: %d", newyres, yshift);
 
+        px1 = xres2 - max_col - kdata.width/2;
+        py1 = yshift - max_row + kdata.height/2 + kdata.y;
         px2 = 0;
         py2 = yshift;
-        px1 = xres2 - max_col - kdata.width/2;
-        py1 = yshift - ((max_row - kdata.height/2) - kdata.y);
-        break;
-
-        case GWY_MERGE_DIRECTION_RIGHT:
-        yshift = MAX(0,  -(max_row - cdata.height/2) - (yres1 - yres2)/2);
-        newxres = cdata.x + (max_col - kdata.width/2) + xres2;
-        newyres = MAX(MAX(yres1, yres2),
-                      (MAX(0, (max_row - cdata.height/2)) + yres2/2) -
-                      (MIN(0, (max_row - cdata.height/2)) - yres2/2));
-        gwy_debug("%d %d %d %d\n",
-               yres1, yres2,
-               (MAX(0, (max_row - cdata.height/2)) + yres2/2),
-               (MIN(0, (max_row - cdata.height/2)) - yres2/2));
-        gwy_debug("newyres: %d\n, yshift: %d", newyres, yshift);
-
-
-        px1 = 0;
-        py1 = yshift;
-        px2 = cdata.x + (max_col - kdata.width/2);
-        py2 = yshift + ((max_row - kdata.height/2) - kdata.y);
         break;
 
         default:
@@ -541,14 +539,8 @@ merge_do(MergeArgs *args)
         break;
     }
 
-
-    gwy_data_field_resample(result, newxres, newyres,
-                            GWY_INTERPOLATION_NONE);
-
-    put_fields(dfield1, dfield2, result, args->boundary,
-               px1, py1,
-               px2, py2);
-
+    gwy_data_field_resample(result, newxres, newyres, GWY_INTERPOLATION_NONE);
+    put_fields(dfield1, dfield2, result, real_boundary, px1, py1, px2, py2);
 
     /* set right output */
     if (result) {
@@ -571,6 +563,74 @@ merge_do(MergeArgs *args)
     return TRUE;
 }
 
+static void
+merge_do_uncorrelated(MergeArgs *args)
+{
+    GwyContainer *data;
+    GwyDataField *dfield1, *dfield2, *result;
+    gint newxres, newyres, xres1, xres2, yres1, yres2;
+    GQuark quark;
+    gint newid;
+    gdouble m;
+
+    quark = gwy_app_get_data_key_for_id(args->op1.id);
+    dfield1 = GWY_DATA_FIELD(gwy_container_get_object(args->op1.data, quark));
+
+    quark = gwy_app_get_data_key_for_id(args->op2.id);
+    dfield2 = GWY_DATA_FIELD(gwy_container_get_object(args->op2.data, quark));
+
+    xres1 = gwy_data_field_get_xres(dfield1);
+    xres2 = gwy_data_field_get_xres(dfield2);
+    yres1 = gwy_data_field_get_yres(dfield1);
+    yres2 = gwy_data_field_get_yres(dfield2);
+    result = gwy_data_field_new_alike(dfield1, FALSE);
+    m = MIN(gwy_data_field_get_min(dfield1), gwy_data_field_get_min(dfield2));
+
+    if (args->direction == GWY_MERGE_DIRECTION_UP
+        || args->direction == GWY_MERGE_DIRECTION_DOWN) {
+        newxres = MAX(xres1, xres2);
+        newyres = yres1 + yres2;
+    }
+    else if (args->direction == GWY_MERGE_DIRECTION_LEFT
+          || args->direction == GWY_MERGE_DIRECTION_RIGHT) {
+        newxres = xres1 + xres2;
+        newyres = MAX(yres1, yres2);
+    }
+    else {
+        g_return_if_reached();
+    }
+
+    gwy_data_field_resample(result, newxres, newyres, GWY_INTERPOLATION_NONE);
+    gwy_data_field_fill(result, m);
+
+    if (args->direction == GWY_MERGE_DIRECTION_UP) {
+        gwy_data_field_area_copy(dfield2, result, 0, 0, xres2, yres2, 0, 0);
+        gwy_data_field_area_copy(dfield1, result, 0, 0, xres1, yres1, 0, yres2);
+    }
+    else if (args->direction == GWY_MERGE_DIRECTION_DOWN) {
+        gwy_data_field_area_copy(dfield1, result, 0, 0, xres1, yres1, 0, 0);
+        gwy_data_field_area_copy(dfield2, result, 0, 0, xres2, yres2, 0, yres1);
+    }
+    else if (args->direction == GWY_MERGE_DIRECTION_LEFT) {
+        gwy_data_field_area_copy(dfield2, result, 0, 0, xres2, yres2, 0, 0);
+        gwy_data_field_area_copy(dfield1, result, 0, 0, xres1, yres1, xres2, 0);
+    }
+    else if (args->direction == GWY_MERGE_DIRECTION_RIGHT) {
+        gwy_data_field_area_copy(dfield1, result, 0, 0, xres1, yres1, 0, 0);
+        gwy_data_field_area_copy(dfield2, result, 0, 0, xres2, yres2, xres1, 0);
+    }
+
+    gwy_app_data_browser_get_current(GWY_APP_CONTAINER, &data, 0);
+    newid = gwy_app_data_browser_add_data_field(result, data, TRUE);
+    gwy_app_set_data_field_title(data, newid, _("Merged images"));
+    gwy_app_sync_data_items(args->op1.data, data, args->op1.id, newid,
+                            FALSE,
+                            GWY_DATA_ITEM_PALETTE,
+                            GWY_DATA_ITEM_MASK_COLOR,
+                            GWY_DATA_ITEM_RANGE,
+                            0);
+    g_object_unref(result);
+}
 
 static void
 put_fields(GwyDataField *dfield1, GwyDataField *dfield2,
@@ -585,6 +645,7 @@ put_fields(GwyDataField *dfield1, GwyDataField *dfield2,
     gint x1, x2, y1_, y2, w1, w2, h1, h2;
     gint xres1, xres2, yres1, yres2;
 
+    gwy_debug("px1: %u, py1: %u, px2: %u, py2: %u", px1, py1, px2, py2);
     xres1 = gwy_data_field_get_xres(dfield1);
     yres1 = gwy_data_field_get_yres(dfield1);
     xres2 = gwy_data_field_get_xres(dfield2);
@@ -608,9 +669,8 @@ put_fields(GwyDataField *dfield1, GwyDataField *dfield2,
     w2 = gwy_data_field_get_xres(dfield2);
     h2 = gwy_data_field_get_yres(dfield2);
 
-
     if (boundary == GWY_MERGE_BOUNDARY_SMOOTH
-        || boundary == GWY_MERGE_BOUNDARY_FIRST) {
+        || boundary == GWY_MERGE_BOUNDARY_SECOND) {
         gwy_data_field_area_copy(dfield1, result,
                                  x1, y1_, w1, h1,
                                  px1, py1);
@@ -641,8 +701,7 @@ put_fields(GwyDataField *dfield1, GwyDataField *dfield2,
 
         if (py1 < py2) {
             res_rect.y = py2;
-            res_rect.height = MAX(MIN(yres1, yres2),
-                                  py1 + yres1 - py2 - (yres1 - yres2));
+            res_rect.height = py1 + yres1 - py2;
         }
         else {
             res_rect.y = py1;
@@ -750,6 +809,14 @@ merge_boundary(GwyDataField *dfield1,
     gint col, row;
     gdouble weight, val1, val2;
 
+    gwy_debug("dfield1: %u x %u at (%u, %u)",
+              dfield1->xres, dfield1->yres, f1_pos.x, f1_pos.y);
+    gwy_debug("dfield2: %u x %u at (%u, %u)",
+              dfield2->xres, dfield2->yres, f2_pos.x, f2_pos.y);
+    gwy_debug("result: %u x %u", result->xres, result->yres);
+    gwy_debug("rect in result : %u x %u at (%u,%u)",
+              res_rect.width, res_rect.height, res_rect.x, res_rect.y);
+
     for (col = 0; col < res_rect.width; col++) {
         for (row = 0; row < res_rect.height; row++) {
 
