@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: obj_synth.c 12108 2011-02-18 21:53:28Z yeti-dn $
+ *  @(#) $Id: obj_synth.c 14942 2013-04-21 12:46:23Z yeti-dn $
  *  Copyright (C) 2009-2011 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include "config.h"
@@ -281,7 +281,7 @@ static const ObjSynthFeature features[] = {
     { OBJ_SYNTH_THATCH,   N_("Thatches"),     &create_thatch,   &getcov_thatch,   },
     { OBJ_SYNTH_TENT,     N_("Tents"),        &create_tent,     &getcov_tent,     },
     { OBJ_SYNTH_GAUSSIAN, N_("Gaussians"),    &create_gaussian, &getcov_gaussian, },
-    { OBJ_SYNTH_DOUGHNUT, N_("Dougnuts"),     &create_doughnut, &getcov_doughnut, },
+    { OBJ_SYNTH_DOUGHNUT, N_("Doughnuts"),    &create_doughnut, &getcov_doughnut, },
 };
 
 static GwyModuleInfo module_info = {
@@ -289,7 +289,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Generates randomly patterned surfaces by placing objects."),
     "Yeti <yeti@gwyddion.net>",
-    "1.3",
+    "1.5",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -386,9 +386,10 @@ run_noninteractive(ObjSynthArgs *args,
     if (!replace) {
         if (data) {
             newid = gwy_app_data_browser_add_data_field(dfield, data, TRUE);
-            gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
-                                    GWY_DATA_ITEM_GRADIENT,
-                                    0);
+            if (oldid != -1)
+                gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
+                                        GWY_DATA_ITEM_GRADIENT,
+                                        0);
         }
         else {
             newid = 0;
@@ -414,7 +415,7 @@ obj_synth_dialog(ObjSynthArgs *args,
                  GwyDataField *dfield_template,
                  gint id)
 {
-    GtkWidget *dialog, *table, *vbox, *hbox, *notebook, *spin;
+    GtkWidget *dialog, *table, *vbox, *hbox, *notebook;
     ObjSynthControls controls;
     GwyDataField *dfield;
     GwyPixmapLayer *layer;
@@ -433,7 +434,6 @@ obj_synth_dialog(ObjSynthArgs *args,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          GTK_STOCK_OK, GTK_RESPONSE_OK,
                                          NULL);
-    gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
     controls.dialog = dialog;
 
@@ -450,11 +450,10 @@ obj_synth_dialog(ObjSynthArgs *args,
                                 dimsargs->measure*PREVIEW_SIZE,
                                 TRUE);
     gwy_container_set_object_by_name(controls.mydata, "/0/data", dfield);
-    if (data)
+    if (dfield_template) {
         gwy_app_sync_data_items(data, controls.mydata, id, 0, FALSE,
                                 GWY_DATA_ITEM_PALETTE,
                                 0);
-    if (dfield_template) {
         controls.surface = gwy_synth_surface_for_preview(dfield_template,
                                                          PREVIEW_SIZE);
         controls.zscale = 3.0*gwy_data_field_get_rms(dfield_template);
@@ -516,8 +515,8 @@ obj_synth_dialog(ObjSynthArgs *args,
     controls.coverage = gtk_adjustment_new(args->coverage,
                                            0.001, 12.0, 0.001, 1.0, 0);
     g_object_set_data(G_OBJECT(controls.coverage), "target", &args->coverage);
-    spin = gwy_table_attach_hscale(table, row, _("Co_verage:"), NULL,
-                                   controls.coverage, GWY_HSCALE_SQRT);
+    gwy_table_attach_hscale(table, row, _("Co_verage:"), NULL,
+                            controls.coverage, GWY_HSCALE_SQRT);
     g_signal_connect_swapped(controls.coverage, "value-changed",
                              G_CALLBACK(gwy_synth_double_changed), &controls);
     row++;
@@ -945,8 +944,8 @@ object_synth_iter(GwyDataField *surface,
         gdouble *p;
 
         id = g_rand_int_range(rngset->rng[RNG_ID], 0, ncells - k);
-        i = indices[id]/nycells;
-        j = indices[id] % nycells;
+        i = indices[id]/nxcells;
+        j = indices[id] % nxcells;
         indices[id] = indices[ncells-1 - k];
 
         size = args->size;
