@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: pat_synth.c 12126 2011-02-26 10:45:26Z yeti-dn $
+ *  @(#) $Id: pat_synth.c 14942 2013-04-21 12:46:23Z yeti-dn $
  *  Copyright (C) 2010,2011 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include "config.h"
@@ -240,7 +240,7 @@ static GwyModuleInfo module_info = {
     N_("Generates surfaces representing simple patterns "
        "(steps, ridges, ...)."),
     "Yeti <yeti@gwyddion.net>",
-    "1.1",
+    "1.3",
     "David Nečas (Yeti)",
     "2010",
 };
@@ -356,9 +356,10 @@ run_noninteractive(PatSynthArgs *args,
     if (!replace) {
         if (data) {
             newid = gwy_app_data_browser_add_data_field(dfield, data, TRUE);
-            gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
-                                    GWY_DATA_ITEM_GRADIENT,
-                                    0);
+            if (oldid != -1)
+                gwy_app_sync_data_items(data, data, oldid, newid, FALSE,
+                                        GWY_DATA_ITEM_GRADIENT,
+                                        0);
         }
         else {
             newid = 0;
@@ -403,7 +404,6 @@ pat_synth_dialog(PatSynthArgs *args,
                                          GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
                                          GTK_STOCK_OK, GTK_RESPONSE_OK,
                                          NULL);
-    gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
     gtk_dialog_set_default_response(GTK_DIALOG(dialog), GTK_RESPONSE_OK);
     controls.dialog = dialog;
 
@@ -420,11 +420,10 @@ pat_synth_dialog(PatSynthArgs *args,
                                 dimsargs->measure*PREVIEW_SIZE,
                                 TRUE);
     gwy_container_set_object_by_name(controls.mydata, "/0/data", dfield);
-    if (data)
+    if (dfield_template) {
         gwy_app_sync_data_items(data, controls.mydata, id, 0, FALSE,
                                 GWY_DATA_ITEM_PALETTE,
                                 0);
-    if (dfield_template) {
         controls.surface = gwy_synth_surface_for_preview(dfield_template,
                                                          PREVIEW_SIZE);
         controls.zscale = 3.0*gwy_data_field_get_rms(dfield_template);
@@ -642,6 +641,7 @@ pattern_type_selected(GtkComboBox *combo,
     gtk_widget_show_all(GTK_WIDGET(controls->table_parameters));
     gtk_widget_show_all(GTK_WIDGET(controls->table_placement));
 
+    update_values(controls);
     pat_synth_invalidate(controls);
 }
 
@@ -932,7 +932,7 @@ make_pattern_steps(const PatSynthArgs *args,
     GwyDataField *displacement_x;
     guint n, i, j, k, xres, yres;
     gdouble *abscissa, *height, *data, *dx_data;
-    gdouble h, c, s, xoff, yoff, range, margin, pitch;
+    gdouble h, c, s, xoff, range, margin, pitch;
 
     h = pargs->height * pow10(dimsargs->zpow10);
 
@@ -971,14 +971,12 @@ make_pattern_steps(const PatSynthArgs *args,
     c = cos(pargs->angle);
     s = sin(pargs->angle);
     xoff = 0.5*((1.0 - c)*xres + s*yres);
-    yoff = 0.5*(-s*yres + (1.0 - c)*yres);
     for (i = 0; i < yres; i++) {
         for (j = 0; j < xres; j++) {
-            gdouble v, x, xu, yu, d, q;
+            gdouble v, x, xu, d, q;
             guint kmod;
 
             xu = xoff + j*c - i*s;
-            yu = yoff + j*s + i*c;
             x = xu + dx_data[i*xres + j];
             k = bisect_lower(abscissa, 2*n, x);
             kmod = k % 2;
@@ -1237,7 +1235,7 @@ make_pattern_ridges(const PatSynthArgs *args,
     GwyDataField *displacement_x;
     guint n, i, j, k, xres, yres;
     gdouble *abscissa, *height, *data, *dx_data;
-    gdouble h, c, s, xoff, yoff, range, margin, pitch;
+    gdouble h, c, s, xoff, range, margin, pitch;
 
     h = pargs->height * pow10(dimsargs->zpow10);
 
@@ -1284,14 +1282,12 @@ make_pattern_ridges(const PatSynthArgs *args,
     c = cos(pargs->angle);
     s = sin(pargs->angle);
     xoff = 0.5*((1.0 - c)*xres + s*yres);
-    yoff = 0.5*(-s*yres + (1.0 - c)*yres);
     for (i = 0; i < yres; i++) {
         for (j = 0; j < xres; j++) {
-            gdouble v, x, xu, yu, d, q;
+            gdouble v, x, xu, d, q;
             guint kmod;
 
             xu = xoff + j*c - i*s;
-            yu = yoff + j*s + i*c;
             x = xu + dx_data[i*xres + j];
             k = bisect_lower(abscissa, 4*n, x);
             kmod = k % 4;
