@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: icolorange.c 11559 2010-11-15 10:47:22Z yeti-dn $
+ *  @(#) $Id: icolorange.c 15203 2013-08-23 18:56:57Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -15,7 +15,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
 #include "config.h"
@@ -117,11 +117,11 @@ static void   gwy_tool_color_range_spin_changed      (GwyToolColorRange *tool);
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
-    N_("Interactive color range tool, allows to select data range false "
+    N_("Interactive color range tool, allows selecting the data range false "
        "color scale should map to, either on data or on height distribution "
        "histogram."),
     "Yeti <yeti@gwyddion.net>",
-    "3.11",
+    "3.12",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
@@ -439,7 +439,7 @@ gwy_tool_color_range_data_switched(GwyTool *gwytool,
     tool->data_switch = FALSE;
     gwy_debug("B");
 
-    if (ignore || plain_tool->init_failed)
+    if (plain_tool->init_failed)
         return;
 
     if (data_view) {
@@ -454,10 +454,15 @@ gwy_tool_color_range_data_switched(GwyTool *gwytool,
         gtk_widget_set_sensitive(GTK_WIDGET(tool->histogram), FALSE);
         gtk_widget_set_sensitive(GTK_WIDGET(tool->spinmin), FALSE);
         gtk_widget_set_sensitive(GTK_WIDGET(tool->spinmax), FALSE);
+        gwy_selection_clear(tool->graph_selection);
     }
 
     tool = GWY_TOOL_COLOR_RANGE(gwytool);
     gwy_tool_color_range_update_histogram(tool);
+
+    if (ignore)
+        return;
+
     range_type = gwy_tool_color_range_get_range_type(tool);
     if (data_view) {
         if (range_type == GWY_LAYER_BASIC_RANGE_FIXED) {
@@ -827,7 +832,7 @@ gwy_tool_color_range_update_histogram(GwyToolColorRange *tool)
     gwy_data_field_dh(plain_tool->data_field, tool->heightdist, 0);
     /* rescale to sqrt to make more readable  */
     gwy_data_line_sqrt(tool->heightdist);
-  
+
     gwy_graph_curve_model_set_data_from_dataline(cmodel, tool->heightdist,
                                                  0, 0);
 }
